#ifndef GRCC_H
#define GRCC_H

#include <map>
#include <vector>
#include <string>
#include <list>
#include "UdmBase.h"
#include "GRCExport.h"

///////////////////////////////////////////////////////////////////////////////
extern bool isGenerationNecessary( const std::string& file, const std::string& dependOnThis);

///////////////////////////////////////////////////////////////////////////////
typedef std::list< std::string> StrList_t;

///////////////////////////////////////////////////////////////////////////////
struct GRCC_DLL CCodeGenConfig
{
  // dependent file (full path)
  // generation will happen only if the last write time of '_depFileName' 
  // is greater than the GRC library, or the GRC library does not exist.
  std::string _depFileName;
  // transformation model (full path)
  std::string _transFileName;
  // udm directory
  std::string _udmDirName;
  // all the names of meta used in GRT
  StrList_t _metas;
  // debug or release
  bool _debug; // true means debug libraries
  // dynamic or static link library
  bool _dynamic;	// true means dynamic link libraries
};

///////////////////////////////////////////////////////////////////////////////
class GRCC_DLL GRCC {
public:
	GRCC( const CCodeGenConfig& ccfg); 
	virtual ~GRCC();
	// generate DLL/static libs. this is the primary interface.
	void generateLibrary();
	// generate AM and guard code 
	// outputs: fn.cpp, fn.h, fn.def, fn-inc.h
	void generateSourceCode( const std::string& fn);
	// outputs: fn_genops.cpp, fn_genops.h
	void generateSpecCmpCode( const std::string& fn);
	// generate MPC file (common project file format)
	void generateMpc( const std::string& fn);

protected:
	int makeLibrary( const std::string& fnamebase);

private:
	CCodeGenConfig _ccfg;
	Udm::SmartDataNetwork _transformation;
//	GR::RootFolder _rf;
};

#endif //GRCC_H
