#ifndef IGReATParameters_h
#define IGReATParameters_h

#include "GReATTypes.h"
#include <string>
#include <vector>
#include <set>
#include <list>

typedef std::vector<std::string> StrVec;
typedef std::list< std::pair< std::string, Uml::Diagram> > StrUmlDiagrams_t;

class IGReATParameters
{
public:
  typedef enum {SUCCESS, HALF_SUCCESS, FAILED} SaveResult;
public:
  virtual ~IGReATParameters() {}
  virtual void open(const std::string& fname) = 0;
  virtual void open(const string &cfgname, const vector<string> &files) = 0;
  virtual void close() = 0;
  virtual void closeNoSave() = 0;
  virtual SaveResult saveProgram() = 0;
  virtual SaveResult saveInternalBackend(bool doException = false) = 0;
  virtual void closeProgram() = 0;
  virtual void closeProgramNoSave() = 0;
  virtual void reloadProgram() = 0;
  virtual void getRootFolder(GR::RootFolder& rootFolder) = 0;
  virtual void getGeneralBackend(GeneralBackendPtr& dataNetworkMap) = 0;
  virtual void getTransformation(UdmDataNetworkPtr& transformation) = 0;
  virtual void getStartRule(GR::RewritingRule& rule) = 0;
  virtual void getPassAlongPackets(PassAlongPackets& agruments) = 0;
  virtual void getOutputMsg(string &outputMsg) = 0;
  virtual std::string getTransfLastWrittenFileName() = 0;
  virtual std::string getUdmDirName() = 0;
  virtual StrUmlDiagrams_t getMetas() = 0;
//  virtual void getUserCodeInfo(StrVec& incFiles,
//                                      StrVec& libFiles,
//                                      StrVec& incDirs,
//                                      StrVec& libDirs) = 0;

  virtual void setCurrentWorkingPath(const string &path) = 0;
  virtual void useMemoryBackend() = 0;
};

#endif //IGReATParameters_h
