// GREngineExampleTest.cpp: implementation of the GREngineExampleTest class.
//
//////////////////////////////////////////////////////////////////////

#include "GREngineExampleTest.h"

#include "GReATFactory.h"
#include "GReAT.h"
#include "GReATParametersFactory.h"
#include "CINTCodeGathererFactory.h"

#include <time.h>
#include <direct.h>
#include <conio.h>

//Config filename and path hardcoded for now, will change later
#define GREAT_CONFIG_FILE "C:\\Mobies\\Great\\Samples\\HOUSE2ORDER\\Config.mga"

// Registers the fixture into the 'registry'

CPPUNIT_TEST_SUITE_REGISTRATION( GREngineExampleTest );

//============================================
class Notify : public IGReAT::ICtrlNotify
{
public:
  static bool m_quitApp;
public:

  // IGReAT::ICtrlNotify
  void reportProgress(double prg) {}
  bool quit()
  {
    return m_quitApp;
  }
};
//============================================
bool Notify::m_quitApp = false;

extern HINSTANCE hCINTDLL;
typedef Udm::UdmDiagram* (*INIT_FN)();
typedef Udm::UdmDiagram* (*MAP_FN)();

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GREngineExampleTest::GREngineExampleTest()
{

}

GREngineExampleTest::~GREngineExampleTest()
{

}

void GREngineExampleTest::testGRE()
{
	//no try or catch, CPPUNIT will catch exceptions, reporting as a test failure

		IGReATParameters* grParams = 0;
		great = 0;
		ICINTCodeGatherer* cintDll = 0;

		char buffer[200];
		_getcwd(buffer, 200);


	CPPUNIT_ASSERT(grParams = GReATParametersFactory::create());
		grParams->setCurrentWorkingPath((string)buffer);

		grParams->open(GREAT_CONFIG_FILE);

		GeneralBackendPtr dataNetworkMap;
		grParams->getGeneralBackend(dataNetworkMap);

		UdmDataNetworkPtr transformation;
		grParams->getTransformation(transformation);

	CPPUNIT_ASSERT(great = GReATFactory::create());

		great->init(dataNetworkMap, transformation);

		Notify notify;
		great->setCtrlNotify(&notify);

	CPPUNIT_ASSERT(cintDll = CINTCodeGathererFactory::create());
	    UserCodeGenConfig cfg;
		cfg.m_depFileName = grParams->getTransfLastWrittenFileName();
		cfg.m_udmDirName = grParams->getUdmDirName();
	CPPUNIT_ASSERT(cfg.m_transf = transformation);
		cfg.m_metas = grParams->getMetas();

		StrVec incFiles, libFiles, incDirs, libDirs;
		grParams->getUserCodeInfo(incFiles, libFiles, incDirs, libDirs);

		cfg.m_incFiles = incFiles;
		cfg.m_libFiles = libFiles;
		cfg.m_incDirs = incDirs;
		cfg.m_libDirs = libDirs;

		#ifdef _DEBUG
		    cfg.m_debug = true;
	    #else
			cfg.m_debug = false;
		#endif

		cintDll->generateDll(cfg);
		HINSTANCE hDll  = cintDll->loadDll();
		great->setCINTDll(hDll);


		GR::RewritingRule startRule;
		PassAlongPackets arguments;
		grParams->getStartRule(startRule);
		grParams->getPassAlongPackets(arguments);

	    if (!Notify::m_quitApp)
	    {
   
				great->Execute(startRule, arguments);

		}
		if (Notify::m_quitApp)
		{
		
				grParams->closeNoSave();
		}
		arguments.clear();
		if(great)
			delete great, great=0;
		grParams->close();
		if(grParams)
			delete grParams, grParams=0;
		if (cintDll)
			delete cintDll, cintDll = 0;

}


