#include <iostream> // for cout
#include <UdmBase.h>
#include "ConfigGenerator.h"
#include "GReATConfigMeta.h"
#include "InputFileRegistry.h"
#include "GReATVersion.h"

UDM_USE_MGA

void printVer()
{
	std::cout << "CodeGenerator v" << GREAT_VERSION << " - GR to C++ compiler utility.\n(c) 2000-2007 Vanderbilt University\nInstitute for Software Integrated Systems\n\n";
}

int usage()
{
	printVer();
	std::cout << "Usage: CG.exe <GenFileName> <ConfigFileName> [-m|-t|-p|-e]\n";
	std::cout << "GenFileName\tthe generated rewriting code.\n";
	std::cout << "ConfigFileName\tGReAT Configuration information file.\n";
	std::cout << "[-m]\tGenerate the main.cpp file only.\n";
	std::cout << "[-t]\tGenerate the translator files only.\n";
	std::cout << "[-p]\tGenerate Visual Studio projects files also.\n";
	std::cout << "[-e]\tGenerate transformation executable file.(Compile transformation)\n\n";
	std::cout << "Examples:" << std::endl;
	std::cout << "CG.exe SF2FSF SF2FSFConfig.mga" << std::endl;
	std::cout << "CG.exe HSM2FSM StateChartConfig.mga -t" << std::endl;
	std::cout << std::endl;
	return -1;
}

/*
int testInputFileRegistry()
{
	InputFileRegistry ifr;
	const std::string test1= " INPUT  = \"models\\open this.xml\"";// ; \"..\\write as this.xml\"";
	const std::string test2= "INPUT=\"models\\open this.xml\";\"..\\write as this.xml\"";
	const std::string test3= "INPUT=\"models\\open this.xml\"!\"..\\write as this.xsd\"";
	const std::string test4= "INPUT=\"models\\open this.xml\";\"..\\write as this.xml\" !      \"..\\write as this.xsd\"";
	const std::string test5= "INPUT=\"models\\open this.xml\"!\"..\\write as this.xsd\"   ;\"..\\write as this.xml\"";
	ifr.registerFile( test1);
	ifr.registerFile( test2);
	ifr.registerFile( test3);
	ifr.registerFile( test4);
	ifr.registerFile( test5);
	return 0;
}
*/
int main( int argc, char* argv[])
{
	// argv[ 1] is code gen file name base
	// argv[ 2] is config file path
	// argv[ 3] is optional, either: <nothing> -OR- -m -OR- -t
	try
	{
		if ( ( argc <3) || (argc >4) || (( argc== 4) && (std::string( argv[ 3]) != "-m") && ( std::string( argv[ 3]) != "-t") && ( std::string( argv[ 3]) != "-p") && ( std::string( argv[ 3]) != "-e")))
			return usage();
		printVer();
		ConfigGenerator cfgGen( argv[ 2]);
		std::string currWD= getPath( argv[ 2]);
		currWD= ( currWD.empty()) ? getCwd() : currWD;
		createLogFile( currWD);
		printLogMsg( "Compiling \'" + getFullPath( currWD, cfgGen.getTransformationFileName() ) +"\'...\n\n");
		// Generate
		cfgGen.generate();
		bool generateMain= ( argc== 3) || ( std::string( argv[ 3])!= "-t");
		bool generateTrans= ( argc== 3) || ( std::string( argv[ 3])!= "-m");
		bool generateProj= ( argc > 3) && ( std::string( argv[ 3])== "-p");
		bool generateExe= ( argc > 3) && ( std::string( argv[ 3])== "-e");
		if ( generateMain)
		{
			cfgGen.generateMain();
			cfgGen.saveMain( 
				getFullPath( currWD, dropExtension( argv[ 1])) + std::string( "_main.cpp"), 
				getFullPath( currWD, dropExtension( argv[ 1]))
				);
		}
		if ( generateTrans)
			cfgGen.save( getFullPath( currWD, dropExtension( argv[ 1])));
		if ( generateProj)
			cfgGen.saveVisualStudioProject( getFullPath( currWD, dropExtension( argv[ 1])));
		if ( generateExe)
			cfgGen.generateExe( getFullPath( currWD, dropExtension( argv[ 1])));
		// Print status
		printLogMsg( "\n----------------------------\nOutput files:\n\t"+ 
			( generateMain ? getFullPath( currWD, dropExtension( std::string( argv[ 1]))) + "_main.cpp" + "\n\t" : "") + 
			( generateTrans ? getFullPath( currWD, std::string( dropExtension( argv[ 1]))) + ".cpp\n\t" + getFullPath( currWD, dropExtension( argv[ 1])) + ".h\n\t" : "") +
			( generateProj ? getFullPath( currWD, dropExtension( std::string( argv[ 1]))) + ".dsp" + "\n\t" : "") + 
			( generateExe ? getFullPath( currWD, dropExtension( std::string( argv[ 1]))) + ".exe" + "\n\t" : "")  
			);
		printLogMsg( num_to_string( g_cntErr) + " error(s), " + num_to_string( g_cntWrn) + " warning(s).\n");
		closeLogFile();
	}
	catch(udm_exception &e)
	{
		cout << e.what() << endl;
		closeLogFile();
	}
	catch(CGEx &e)
	{
		closeLogFile();
	}
	return 0;
}
