#include "GroupEffector.h"
#include "CIntCompiler.h"
#include "Logging.h"
#include "PEStringTable.h"
#include "UmlExt.h"
#include "GReATUtils.h"
#include "OutputSorter.h"


GroupEffector::GroupEffector( const GRSimpleRule& rule, 
		const GeneralBackend& backend, 
		std::map< GR::ObjectWrapper, std::string>& boundObjects,
		ICIntCompiler& cc,
		const ParamDatas_t& ruleOutputs
		): Effector(rule, backend, boundObjects, cc) 
{
	const set<GR::PatternGroup> groups = _rule.PatternGroup_kind_children();
	_group = *(groups.begin());
	_ruleOutputs = ruleOutputs;

	const set<GR::GRBase>& groupMembers = _group.members();
	for(set<GR::GRBase>::const_iterator i=groupMembers.begin();i!=groupMembers.end();++i)
	{		
		const GR::GRBase& grBase = *i;
		if((std::string)grBase.type().name() == "PatternObject")
		{
			const GR::ObjectWrapper& grObj = GR::ObjectWrapper::Cast(grBase);
			if(grObj.isAssociationClass()) _groupAssocObjs.insert(grObj);
			else 
			{
	//			Uml::Class clsType = Uml::classByName(_backend.getDiagram((std::string)grObj.paradigmName()), (std::string)grObj.classType());				
				Uml::Class clsType;
				string ns_name = grObj.nameSpace();
				if(ns_name.empty())
					clsType = Uml::classByName(_backend.getDiagram((std::string)grObj.paradigmName()), (std::string)grObj.classType());
				else
					clsType = Uml::classByPath(_backend.getDiagram((std::string)grObj.paradigmName()), ns_name+"::"+(std::string)grObj.classType());

				if((string)clsType.stereotype()=="Set") _groupSetObjs.insert(grObj);
				else _groupObjs.insert(grObj);
			}
			continue;
		}
		if((std::string)grBase.type().name() == "ObjectLink")
		{
			const GR::ObjectLink& grLink = GR::ObjectLink::Cast(grBase);
			if((std::string)grLink.relationshipType()=="SimpleAssociation") 
				_groupLinks.insert(grLink);
		}		
	}	

	const set<GR::ObjectWrapper>& grObjs = _rule.ObjectWrapper_kind_children();
	for(set<GR::ObjectWrapper>::const_iterator itObj=grObjs.begin();itObj!=grObjs.end();++itObj)
	{
		const GR::ObjectWrapper& grObj = *itObj;
		if((std::string)grObj.type().name()!="PatternGroup" && (std::string)grObj.action()=="Delete") 
			_deleteObjs.insert(grObj);
	}
	const set<GR::ObjectLink>& grLinks = _rule.ObjectLink_kind_children();
	for(set<GR::ObjectLink>::const_iterator itLink=grLinks.begin();itLink!=grLinks.end();++itLink)
	{
		const GR::ObjectLink& grLink = *itLink;
		if((std::string)grLink.action()=="CreateNew") 
		{
			_newLinks.insert(grLink);
			continue;
		}
		if((std::string)grLink.action()=="Delete")
		{
			if(groupMembers.find(grLink)== groupMembers.end()) _preDeleteLinks.insert(grLink);
			else if((std::string)_group.action()!="Delete")
			{
				_postDeleteLinks.insert(grLink);
			}
			else
			{
				const GR::ObjectWrapper& srcObj = grLink.src_end();
				const GR::ObjectWrapper& dstObj = grLink.dst_end();
				if((groupMembers.find(srcObj)==groupMembers.end() || groupMembers.find(dstObj)==groupMembers.end())) 
					_postDeleteLinks.insert(grLink);
			}
		}		
	}
}
void GroupEffector::executeRightSide( PEComposite& code)
{
	_currCode= &code;
	
	If* isEmpty = new If;
	isEmpty->_expression = "_matches.empty()";
	isEmpty->add(*new Expression("return"));
	_currCode->add(*isEmpty);
	_currCode->add(*new Expression("std::list<Match> match_list"));
	_currCode->add(*new Expression("std::list<Match>::iterator itThe = _matches.begin()"));
	_currCode->add(*new Expression("match_list.push_back(*itThe)"));
	_currCode->add(*new Expression("_groupMatches.push_back(match_list)"));
	_currCode->add(*new Expression("itThe++"));
	Loop* the_matchLoop = new Loop;
	the_matchLoop->_from = "";
	the_matchLoop->_to = "itThe!=_matches.end()";
	the_matchLoop->_increment = "++itThe";
	the_matchLoop->add(*new Expression("Match& the_match = *itThe"));
	the_matchLoop->add(*new Expression("bool found = false"));
	Loop* groupLoop = new Loop;
	groupLoop->createIteratorLoop("itGroup","_groupMatches","std::list< std::list<Match> >",false,false);
	Loop* other_matchLoop = new Loop;
	other_matchLoop->createIteratorLoop("itOther","(*itGroup)","std::list<Match>",false,false);
	other_matchLoop->add(*new Expression("Match& other_match = *itOther"));
	If* isGroupTrue = new If;
	isGroupTrue->_expression = "isGroupTrue(the_match, other_match)";
	isGroupTrue->add(*new Expression("found = true"));
	isGroupTrue->add(*new Expression("(*itGroup).push_back(the_match)"));
	isGroupTrue->add(*new Expression("break"));
	other_matchLoop->add(*isGroupTrue);
	If* isFound = new If;
	isFound->_expression = "found";
	isFound->add(*new Expression("break"));
	groupLoop->add(*isFound);
	groupLoop->add(*other_matchLoop);
	the_matchLoop->add(*groupLoop);
	If* notFound = new If;
	notFound->_expression = "!found";
	notFound->add(*new Expression("std::list<Match> newgroup"));
	notFound->add(*new Expression("newgroup.push_back(the_match)"));
	notFound->add(*new Expression("_groupMatches.push_back(newgroup)"));
	the_matchLoop->add(*notFound);
	_currCode->add(*the_matchLoop);
	
	Loop* egLoop = new Loop;
	egLoop->createIteratorLoop("currGroup","_groupMatches","std::list< std::list<Match> >",false,false);
	egLoop->add(*new Expression("executeGroup(*currGroup)"));
	_currCode->add(*egLoop);
}

void GroupEffector::makeGroupRule( Class& ruleDef)
{
	//Create the isGroupTrue function
	Function* groupFuncDef = new Function;
	groupFuncDef->_name = "isGroupTrue";
	ruleDef.addPrivate( *groupFuncDef);
	groupFuncDef->_returnValue = BOL;	
	groupFuncDef->insertInputParam(FuncParam_t( std::make_pair( "const Match&", "the_match")));
	groupFuncDef->insertInputParam(FuncParam_t( std::make_pair( "const Match&", "other_match")));

	_cc.init( _rule, _boundObjects, Class() /*unused*/);
	_cc.createGroupRule( *groupFuncDef, _group);
}

void GroupEffector::executeGroup( Class& ruleDef)
{
	//Create the executeGroup function
	_groupEffecDef = new Function;
	_groupEffecDef->_name = "executeGroup";
	ruleDef.addPrivate( *_groupEffecDef);
	_groupEffecDef->insertInputParam(FuncParam_t( std::make_pair( "std::list<Match>&", "group_matches")));

	std::map<GR::ObjectWrapper, std::string>::const_iterator it;
	for( it = _boundObjects.begin();it!=_boundObjects.end();++it)
	{
		const GR::ObjectWrapper& currOW = (*it).first;
		std::string variable= (*it).second;
		variable.erase(0,10);

		Definition* currObjList= new Definition;
//		currObjList->_type= "std::list<"+( std::string) currOW.paradigmName()+ RES+ ( std::string) currOW.classType()+">";
		currObjList->_type= "std::list<"+PE::getFullType(currOW)+">";
		currObjList->_variable= variable+"_list";
		_boundObjList[currOW] = currObjList->_variable;
		_groupEffecDef->add( *currObjList);
	}
	
	Loop* matchLoop= new Loop;
	matchLoop->createIteratorLoop( IT+ MATCH, "group_matches", "std::list<"+ SEP+ MATCH+ ">");
	Expression *currMatch = new Expression;	
	currMatch->_expression = "Match currMatch = *itMatch;";
	matchLoop->add(*currMatch);

	for( it = _boundObjects.begin();it!=_boundObjects.end();++it)
	{
		const GR::ObjectWrapper& currOW = (*it).first;
		std::string variable= (*it).second;
		Expression *currList = new Expression;
		currList->_expression = _boundObjList[currOW]+".push_back("+variable+")";
		matchLoop->add( *currList);
	}

	_groupEffecDef->add( *matchLoop);	

	createObjects();
	bindRemainingObjectLinks();
	callAttributeMapping(ruleDef);
	preDeleteObjectLink(ruleDef);
	executeGroupAction(ruleDef);
	postDelete(ruleDef);
	
	//add RTT here 01/06/10
	createRTT();

	//Append the output packet
	bool allOutputsAreInputs= PacketPassing::allOutputsAreInputs( _ruleOutputs);
	bool areOutputsToBeSorted= OutputSorter::areOutputsToBeSorted( _ruleOutputs);
	bool canForwardInputs= allOutputsAreInputs && !areOutputsToBeSorted;
	if ( canForwardInputs) return;
	for( ParamDatas_t::const_iterator itOutputs= _ruleOutputs.begin(); itOutputs!= _ruleOutputs.end(); ++itOutputs)
	{
		std::map<GR::ObjectWrapper, std::string>::const_iterator itBound = _boundObjList.find( (*itOutputs)._param);
		if ( itBound == _boundObjList.end())
		{
			throw CGEx("FATAL: CG internal error. Output parameter: "+ ( std::string)(*itOutputs)._param.name()+ " not bound in output filter.");
		}
		const std::string& listName = (*itBound).second;
		Expression* appendOutput= new Expression;
		const string& outputMemberName= (*itOutputs)._memberDef->_variable;
		appendOutput->_expression = outputMemberName+"->insert("+outputMemberName+"->end(), "+listName+".begin(), "+listName+".end())";
		_groupEffecDef->add(*appendOutput);
	}
}

void GroupEffector::createOutputObject( const GR::ObjectWrapper& parentOW, GR::ObjectWrapper& childOW, const std::string& childRole)
{
	std::string childVar = lowerBegin(PE::createUniqueName((std::string)childOW.name()));

//	std::string parentType = (std::string)parentOW.paradigmName()+ RES+ (std::string)parentOW.classType();
//	std::string childType = (std::string)childOW.paradigmName()+ RES+ (std::string)childOW.classType();
	std::string parentType = PE::getFullType(parentOW);
	std::string childType = PE::getFullType(childOW);
	
	std::string container = _boundObjList[parentOW];
	Definition* singlePObj = new Definition;
	singlePObj->_type = parentType;
	singlePObj->_variable = lowerBegin(PE::createUniqueName("only_"+(std::string)parentOW.name()));
	singlePObj->_initValue = "*("+container+".begin())";
	_groupEffecDef->add(*singlePObj);

	////check whether the parent objets size is 1
	//If* issize1 = new If;
	//issize1->_expression = container+".size()>1";
	//issize1->add(*new Expression("throw udm_exception(\"Error: The size of parent objects for new created objects is greater than one.\")"));
	//_groupEffecDef->add(*issize1);

	Definition* newChildList = new Definition;
	newChildList->_type= "std::list<"+childType+">";
	newChildList->_variable= childVar+"_list";
	_groupEffecDef->add( *newChildList);
	_boundObjList[childOW] = newChildList->_variable;

	Definition* parent2childMap = new Definition;
	parent2childMap->_type= "std::map<"+parentType+", "+childType+">";
	parent2childMap->_variable= lowerBegin(PE::createUniqueName((std::string)parentOW.name()+"2"+(std::string)childOW.name()));
	_groupEffecDef->add( *parent2childMap);

	std::string itname = lowerBegin(PE::createUniqueName("itList"));
	Loop* loop= new Loop;
	loop->createIteratorLoop(itname, container, "std::list<"+parentType+">");
	//
	Definition* currParent = new Definition;
	currParent->_type = CST+SEP+parentType+REF;
	currParent->_variable = lowerBegin(PE::createUniqueName("curr"+(std::string)parentOW.name()));
	currParent->_initValue = PTR + itname;
	loop->add(*currParent);

	If* isSingle = new If;
	isSingle->_expression = currParent->_variable+" != "+singlePObj->_variable;
	isSingle->add(*new Expression("throw udm_exception(\"Error: The size of parent objects for new created objects is greater than one.\")"));
	loop->add(*isSingle);

	//
	Definition* currChild = new Definition;
	currChild->_type = childType;
	currChild->_variable = lowerBegin(PE::createUniqueName("curr"+(std::string)childOW.name()));
	loop->add(*currChild);

	Expression* itmap = new Expression;
	std::string itvar = lowerBegin(PE::createUniqueName("itmap"));
	itmap->_expression = parent2childMap->_type+RES+CIT+SEP+itvar+SEP+EQ+SEP+parent2childMap->_variable+".find("+currParent->_variable+")";
	loop->add(*itmap);
	
	If* ifblock = new If;
	ifblock->_expression = itvar+" == "+parent2childMap->_variable+".end()";
	Expression* createnew = new Expression;
	createnew->_expression = currChild->_variable+SEP+EQ+SEP+childType+RES+"Create("+currParent->_variable+")";
	ifblock->add(*createnew);
	Expression* insertMap = new Expression;
	insertMap->_expression = parent2childMap->_variable+"["+currParent->_variable+"] = "+currChild->_variable;
	ifblock->add(*insertMap);
	Expression* getChild = new Expression;
	getChild->_expression = currChild->_variable+SEP+EQ+SEP+"("+PTR+itvar+").second";
	ifblock->addElse(*getChild);

	loop->add(*ifblock);

	Expression* pushback = new Expression;
	pushback->_expression = newChildList->_variable+".push_back("+currChild->_variable+")";
	loop->add(*pushback);

	_groupEffecDef->add(*loop);

	//in order to match the format of other names, currMatch. add as the prefix to the variable name 
	_boundObjects[childOW]= CURRM+ "."+ childVar;
	childOW.objectID()= 1;
}

void GroupEffector::bindRemainingObjectLinks()
{
	for( set< GR::ObjectLink>::const_iterator itLink= _newLinks.begin(); itLink!= _newLinks.end(); ++itLink)
	{
		createNewObjectLink(*itLink);
	}
}
void GroupEffector::createNewObjectLink( const GR::ObjectLink& linkToCreate)
{
	if( "Composition"== ( std::string) linkToCreate.relationshipType())
		return;	// compositions are processed already, see 'getCompositionToCreate()'.
	GR::ObjectWrapper srcOW= linkToCreate.src_end();
	GR::ObjectWrapper dstOW= linkToCreate.dst_end();
	std::string linkType= linkToCreate.relationshipType();

	std::string srcObjList = _boundObjList[srcOW];
	std::string dstObjList = _boundObjList[dstOW];
	
//	std::string srcType = (std::string)srcOW.paradigmName()+ RES+ (std::string)srcOW.classType();
//	std::string dstType = (std::string)dstOW.paradigmName()+ RES+ (std::string)dstOW.classType();
	std::string srcType = PE::getFullType(srcOW);
	std::string dstType = PE::getFullType(dstOW);

	Definition* itsrc = new Definition;
	itsrc->_type = "std::list<"+srcType+">::iterator";
	itsrc->_variable = lowerBegin(PE::createUniqueName("itsrc"));
	itsrc->_initValue = srcObjList+".begin()";
	_groupEffecDef->add(*itsrc);
	Definition* itdst = new Definition;
	itdst->_type = "std::list<"+dstType+">::iterator";
	itdst->_variable = lowerBegin(PE::createUniqueName("itdst"));
	itdst->_initValue = dstObjList+".begin()";
	_groupEffecDef->add(*itdst);

	While* whileloop = new While;
	whileloop->_condition = itsrc->_variable+NEQ+srcObjList+".end() && "+itdst->_variable+NEQ+dstObjList+".end()";
	Definition* srcObj = new Definition;
	srcObj->_type = srcType+REF;
	srcObj->_variable = lowerBegin(PE::createUniqueName("srcObj"));
	srcObj->_initValue = PTR+itsrc->_variable;
	whileloop->add(*srcObj);
	Definition* dstObj = new Definition;
	dstObj->_type = dstType+REF;
	dstObj->_variable = lowerBegin(PE::createUniqueName("dstObj"));
	dstObj->_initValue = PTR+itdst->_variable;
	whileloop->add(*dstObj);
	// Create code.
	if ( srcOW.isAssociationClass() && ( linkType=="AssociationClass"))
	{
		Definition* assignDstEnd= new Definition;
		assignDstEnd->_variable= srcObj->_variable+ "."+ ( std::string)linkToCreate.dstRoleName()+ "_end()";
		assignDstEnd->_initValue= dstObj->_variable;
		whileloop->add( *assignDstEnd);
	}
	else if  ( dstOW.isAssociationClass() && ( linkType=="AssociationClass"))
	{
		Definition* assignSrcEnd= new Definition;
		assignSrcEnd->_variable= dstObj->_variable+ "."+ ( std::string)linkToCreate.srcRoleName()+ "_end()";
		assignSrcEnd->_initValue= srcObj->_variable;
		whileloop->add( *assignSrcEnd);
	}
	else // none of srcOW and dstOW are association classes.
	{
		bool isMulCardinality= ( 1 < 
			(unsigned int) getAssociationCardinality( ( std::string)srcOW.paradigmName(), (std::string)srcOW.nameSpace(), ( std::string)srcOW.classType(), ( std::string)linkToCreate.dstRoleName(), false));
		Expression* newLink= new Expression;
		newLink->_expression= srcObj->_variable+ "."+ ( std::string)linkToCreate.dstRoleName()+ "()"+
			(isMulCardinality ? PEQ : EQ)+ SEP+ dstObj->_variable;
		whileloop->add( *newLink);
	}
	
	Expression* itsrcp = new Expression;
	itsrcp->_expression = itsrc->_variable+INC;
	whileloop->add(*itsrcp);
	Expression* itdstp = new Expression;
	itdstp->_expression = itdst->_variable+INC;
	whileloop->add(*itdstp);

	_groupEffecDef->add(*whileloop);
}

void GroupEffector::callAttributeMapping( Class& ruleDef)
{
	set< GR::Code> codes= _rule.Code_kind_children();
	if (codes.empty()) return;
	
	Function* executeAM = new Function;
	executeAM->_name = "callAttributeMapping";
	std::string arglists;
	for(std::map<GR::ObjectWrapper, std::string>::const_iterator i=_boundObjList.begin();i!=_boundObjList.end();++i)
	{	
		const GR::ObjectWrapper& grObj = (*i).first;
		//consider the uniqueness of the grObj
		arglists.append(","+(*i).second);
//		std::string argListType = "std::list<"+(std::string)grObj.paradigmName()+ RES+ (std::string)grObj.classType()+">"+REF;
		std::string argListType = "std::list<"+PE::getFullType(grObj)+">"+REF;

		executeAM->insertInputParam(FuncParam_t( std::make_pair( argListType, (std::string)grObj.name())));
	}
	for(set<GR::Code>::const_iterator ci=codes.begin();ci!=codes.end();++ci)
	{
		executeAM->add( *new Expression( "{"+ ( std::string) (*ci).code()+ "}"));
	}
	ruleDef.addPrivate( *executeAM);
	
	if(arglists.size()>1) arglists.erase(0, 1);
	Expression* callAMFunc = new Expression;
	callAMFunc->_expression = executeAM->_name+"("+arglists+")";
	_groupEffecDef->add(*callAMFunc);
}

void GroupEffector::preDeleteObjectLink(Class& ruleDef)
{
	if(_preDeleteLinks.empty()) return;
	Function* preDeleteFunc = new Function;
	preDeleteFunc->_name = "preDelete";

	set<GR::ObjectWrapper> delObjs;
	makeDeleteFunc(preDeleteFunc, delObjs, _preDeleteLinks);
	
	ruleDef.addPrivate( *preDeleteFunc);
	FunctionCall* delFuncCaller= new FunctionCall;
	delFuncCaller->_func= preDeleteFunc;
	delFuncCaller->copyParamNames();
	_groupEffecDef->add( *delFuncCaller);
}

void GroupEffector::executeGroupAction(Class& ruleDef)
{
	std::string gaction = _group.action();
	if(gaction == "Bound") return;

	set<GR::GRBase> groupmembers = _group.members();
	if(gaction == "Move")
	{
		moveGroup(ruleDef, groupmembers);
	}
	else if(gaction == "Copy")
	{
		copyGroup(ruleDef, groupmembers);
	}
	else //Delete
	{
		deleteGroup(ruleDef, groupmembers);
	}
}

void GroupEffector::moveGroup(Class& ruleDef, set<GR::GRBase>& groupMembers)
{
	set<GR::ObjectLink> nps = _group.dst();
	if(nps.empty())
		throw udm_exception("In the rule: "+(std::string)_rule.name()+" the Group has no move destination.\n");
	if(nps.size()>1)
		throw udm_exception("In the rule: "+(std::string)_rule.name()+" the Group has more than one move destinations.\n");

	const GR::ObjectLink& np = *(nps.begin());
	const GR::ObjectWrapper& mp = np.dst_end();

	Function* moveFunc = new Function;
	moveFunc->_name = "moveGroup";
	std::string argListType = "std::list<"+PE::getFullType(mp)+">"+REF;
	moveFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[mp])));
	
	moveFunc->add(*new CommentLine("get the single target parent obejct"));
	If* isempty = new If;
	isempty->createIfExprThenReturn(_boundObjList[mp]+".empty()");
	moveFunc->add(*isempty);
	Definition* target = new Definition;
	target->_type = CST+SEP+PE::getFullType(mp)+REF;
	target->_variable = lowerBegin(PE::createUniqueName(((std::string)mp.name())+"_parent"));
	target->_initValue = "*("+_boundObjList[mp]+".begin())";
	moveFunc->add(*target);
	
	map<GR::ObjectWrapper, Definition* > newAssocContainerMap;

	if(!_groupAssocObjs.empty())
	{
		moveFunc->add(*new CommentLine("create association class object mirror imaging in the target"));
		for(set<GR::ObjectWrapper>::const_iterator aci=_groupAssocObjs.begin();aci!=_groupAssocObjs.end();++aci)
		{
			const GR::ObjectWrapper& grAssoc = *aci;
			std::string assocType = PE::getFullType(grAssoc);
			
			moveFunc->insertInputParam(FuncParam_t( std::make_pair("std::list<"+assocType+">"+REF, _boundObjList[grAssoc])));

			Definition* newAssocMap = new Definition;	
			newAssocMap->_type = "map<"+assocType+","+assocType+">";
			newAssocMap->_variable = lowerBegin(PE::createUniqueName("new"+(std::string)grAssoc.name()+"_map"));
			moveFunc->add(*newAssocMap);
			
			GR::ObjectWrapper srcCls, dstCls;
			if(!getAssociationClassEnd(grAssoc, srcCls, dstCls)) continue;
			Definition* newAssocObjs_with_peer_end = new Definition;	
			newAssocObjs_with_peer_end->_type = "map<"+assocType+", pair<"+PE::getFullType(srcCls)+", "+PE::getFullType(dstCls)+"> >";
			newAssocObjs_with_peer_end->_variable = lowerBegin(PE::createUniqueName("new"+(std::string)grAssoc.name()+"_with_peer_end"));
			moveFunc->add(*newAssocObjs_with_peer_end);
			newAssocContainerMap[grAssoc] = newAssocObjs_with_peer_end;
			
			Loop* assocClassLoop= new Loop;
			std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)grAssoc.name()));
			assocClassLoop->createIteratorLoop(itname, _boundObjList[grAssoc], "std::list<"+assocType+">");
			//For example: 
			//const GS_SignalFlowGroup::Signal& currSignal_46= *itSignal_45;
			Definition* currAssoc = new Definition;
			currAssoc->_type = CST+SEP+assocType+REF;
			currAssoc->_variable = lowerBegin(PE::createUniqueName("curr"+(std::string)grAssoc.name()));
			currAssoc->_initValue = PTR+itname;
			assocClassLoop->add(*currAssoc);
			//Check the multiplicity
			If* checkMap = new If;
			checkMap->createIfExprThenContinue(newAssocMap->_variable+".find("+currAssoc->_variable+")!="+newAssocMap->_variable+".end()");
			assocClassLoop->add(*checkMap);
			//For example:
			//GS_SignalFlowGroup::Signal newSignal_47= GS_SignalFlowGroup::Signal::Create(newCC_42_parent);
			Definition* newAssoc = new Definition;
			newAssoc->_type = assocType;
			newAssoc->_variable = lowerBegin(PE::createUniqueName("new"+(std::string)grAssoc.name()));
			newAssoc->_initValue = assocType+RES+"Create("+target->_variable+")";
			assocClassLoop->add(*newAssoc);

			createUdmCopy(assocClassLoop,currAssoc->_variable,newAssoc->_variable);
			//copy attribute
			Expression* insertMap = new Expression;
			insertMap->_expression = newAssocMap->_variable+"["+currAssoc->_variable+"] = "+newAssoc->_variable;
			assocClassLoop->add(*insertMap);
			
			Expression* insertPeerEnds = new Expression;
			std::string srname, drname;
			getAssociationRolenames(grAssoc, srname, drname);
			
			insertPeerEnds->_expression = newAssocObjs_with_peer_end->_variable+"["+newAssoc->_variable+"] = ";
			insertPeerEnds->_expression.append("make_pair("+PE::getFullType(srcCls)+"::Cast("+currAssoc->_variable+"."+srname+"_end()), ");
			insertPeerEnds->_expression.append(PE::getFullType(dstCls)+"::Cast("+currAssoc->_variable+"."+drname+"_end()))");
			assocClassLoop->add(*insertPeerEnds);

			moveFunc->add( *assocClassLoop);			
		}
	}

	//move the objects to the new parent
	list<GR::ObjectWrapper> grObjects;
	grObjects.insert(grObjects.begin(), _groupObjs.begin(), _groupObjs.end());
	grObjects.insert(grObjects.end(), _groupSetObjs.begin(), _groupSetObjs.end());

	for(list<GR::ObjectWrapper>::const_iterator itGRObj=grObjects.begin();itGRObj!=grObjects.end();++itGRObj)
	{
		const GR::ObjectWrapper& tbmoved_obj = *itGRObj;
		std::string objType = PE::getFullType(tbmoved_obj);

		moveFunc->insertInputParam(FuncParam_t( std::make_pair("std::list<"+objType+">"+REF, _boundObjList[tbmoved_obj])));

		moveFunc->add(*new CommentLine("move the objects of type "+objType));
		Loop* movedObjLoop= new Loop;
		std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)tbmoved_obj.name()));
		movedObjLoop->createIteratorLoop(itname, _boundObjList[tbmoved_obj], "std::list<"+objType+">"); 
		Definition* currObj = new Definition;
		currObj->_type = objType;
		currObj->_variable = lowerBegin(PE::createUniqueName("curr"+(std::string)tbmoved_obj.name()));
		currObj->_initValue = PTR+itname;
		movedObjLoop->add(*currObj);
		Expression* moveObj = new Expression;
		moveObj->_expression = currObj->_variable+".parent() = "+target->_variable;
		movedObjLoop->add(*moveObj);
		moveFunc->add(*movedObjLoop);
	}

	//Reset the association
	for(map<GR::ObjectWrapper, Definition* >::const_iterator itmap=newAssocContainerMap.begin();itmap!=newAssocContainerMap.end();++itmap)
	{
		const GR::ObjectWrapper& grnewAssoc = (*itmap).first;
		moveFunc->add(*new CommentLine("reconnect the association"));
		Definition* mapdef = (*itmap).second;
		Loop* moveAssoc = new Loop;
		std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)grnewAssoc.name()));
		moveAssoc->createIteratorLoop(itname, mapdef->_variable, mapdef->_type);
		std::string srname, drname;
		getAssociationRolenames(grnewAssoc, srname, drname);
		Expression* connsrc = new Expression;
		connsrc->_expression = "("+PTR+itname+").first."+srname+"_end() = "+"("+PTR+itname+").second.first";
		moveAssoc->add(*connsrc);
		Expression* conndst = new Expression;
		conndst->_expression = "("+PTR+itname+").first."+drname+"_end() = "+"("+PTR+itname+").second.second";
		moveAssoc->add(*conndst);
		moveFunc->add(*moveAssoc);
	}
	//Delete the association for XML, MEM backend
	for(set<GR::ObjectWrapper>::const_iterator itdel=_groupAssocObjs.begin();itdel!=_groupAssocObjs.end();++itdel)
	{
		const GR::ObjectWrapper& delobj = *itdel;
		std::string delobjType = PE::getFullType(delobj);
		std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)delobj.name()));
		Loop* delobjloop = new Loop;
		delobjloop->createIteratorLoop(itname,_boundObjList[delobj],"std::list<"+delobjType+">");
		Definition* delobjdef = new Definition;
		delobjdef->_type = delobjType;
		delobjdef->_variable = lowerBegin(PE::createUniqueName("curr"+(std::string)delobj.name()));
		delobjdef->_initValue = PTR+itname;
		delobjloop->add(*delobjdef);
		If* ifobjnull = new If;
		ifobjnull->_expression = delobjdef->_variable+SEP+NEQ+SEP+"Udm::null";
		Expression* delobjaction = new Expression;
		delobjaction->_expression = delobjdef->_variable+".DeleteObject()";
		ifobjnull->add(*delobjaction);
		delobjloop->add(*ifobjnull);
		moveFunc->add(*delobjloop);
	}

	ruleDef.addPrivate( *moveFunc);
	FunctionCall* moveFuncCaller= new FunctionCall;
	moveFuncCaller->_func= moveFunc;
	moveFuncCaller->copyParamNames();
	_groupEffecDef->add( *moveFuncCaller);
}

void GroupEffector::createUdmCopy(PE* currCode, std::string fromObj, std::string toObj, bool copyAttribute)
{
	Definition* copy_map = new Definition;
	copy_map->_type = "UdmUtil::copy_assoc_map";
	copy_map->_variable = lowerBegin(PE::createUniqueName("copy_map"));
	currCode->add(*copy_map);
	Expression* udmcopy = new Expression;
	udmcopy->_expression = "UdmUtil::CopyObjectHierarchy("+fromObj+".__impl(), "+toObj+".__impl(), "+toObj+".__impl()->__getdn(),"+copy_map->_variable+")";
	currCode->add(*udmcopy);
	if(copyAttribute)
	{
		Expression* udmcopyattribute = new Expression;
		udmcopyattribute->_expression = toObj+".__impl()->CopyAttributesFrom("+fromObj+".__impl())";
		currCode->add(*udmcopyattribute);
	}
}

//std::string GroupEffector::getPatternObjectType(const GR::ObjectWrapper& grobj)
//{
////	return (std::string)grobj.paradigmName()+ RES+ (std::string)grobj.classType();
//	return (PE::getFullType(grobj));
//}

bool GroupEffector::getAssociationRolenames(const GR::ObjectWrapper& assocCls, std::string& srcRolename, std::string& dstRolename)
{
	set<GR::ObjectLink> srcLinks = assocCls.src();
	if(srcLinks.empty()) throw udm_exception("Association Error.");
	GR::ObjectLink srcLink = *(srcLinks.begin());
	srcRolename = srcLink.srcRoleName();
	dstRolename = srcLink.dstRoleName();
	return true;
}

bool GroupEffector::getAssociationClassEnd(const GR::ObjectWrapper& assocCls, GR::ObjectWrapper& srcCls, GR::ObjectWrapper& dstCls)
{
	GR::ObjectLink srcLink, dstLink;
	if(getAssociationLinks(assocCls, srcLink, dstLink))
	{
		srcCls = srcLink.src_end();
		dstCls = dstLink.dst_end();
		return true;
	}
	else return false;
}

bool GroupEffector::getAssociationLinks(const GR::ObjectWrapper& assocCls, GR::ObjectLink& srcLink, GR::ObjectLink& dstLink)
{
	set<GR::ObjectLink> srcLinks = assocCls.src();
	if(srcLinks.empty()) return false;
	srcLink = *(srcLinks.begin());
	
	set<GR::ObjectLink> dstLinks = assocCls.dst();
	ASSERT(!dstLinks.empty());
	for(set<GR::ObjectLink>::iterator dlinks_it=dstLinks.begin();dlinks_it!=dstLinks.end();++dlinks_it)
	{
		if((string)((*dlinks_it).name())=="Dst") 
		{
			dstLink = *dlinks_it;
			return true;
		}
	}
	return false;
}

void GroupEffector::copyGroup(Class& ruleDef, set<GR::GRBase>& groupMembers)
{
	const set<GR::ObjectLink>& nps = _group.dst();
	if(nps.empty())
		throw udm_exception("In the rule: "+(std::string)_rule.name()+" the Group has no copy destination.\n");
	
	Function* assocMapFunc = new Function;
	makeUdmcopyAssocMapFunc(assocMapFunc);
	ruleDef.addPrivate(*assocMapFunc);

	//Create a template function to copy the object
	set<GR::ObjectWrapper> newParentObjs;
	//set<GR::ObjectWrapper> grObjects;
	//set<GR::ObjectWrapper> grAssocCls;
	//set<GR::ObjectLink> grLinks;

	Function* copyFunc = new Function;
	copyFunc->_name = "copyGroup";
	std::string argListType;
	for(set<GR::ObjectLink>::const_iterator itlink=nps.begin();itlink!=nps.end();++itlink)
	{
		const GR::ObjectWrapper& dstObj = (*itlink).dst_end();
		newParentObjs.insert(dstObj);
		argListType = "std::list<"+PE::getFullType(dstObj)+">"+REF;
		copyFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[dstObj])));
	}
	
	set<GR::ObjectWrapper> grObjects = _groupObjs;
	grObjects.insert(_groupSetObjs.begin(), _groupSetObjs.end());
	grObjects.insert(_groupAssocObjs.begin(), _groupAssocObjs.end());
	for(set<GR::ObjectWrapper>::const_iterator itObj=grObjects.begin();itObj!=grObjects.end();++itObj)
	{
		const GR::ObjectWrapper& grObj = *itObj;
		argListType = "std::list<"+PE::getFullType(grObj)+">"+REF;
		copyFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[grObj])));
	}
	for(set<GR::ObjectLink>::const_iterator itLink=_groupLinks.begin();itLink!=_groupLinks.end();++itLink)
	{
		const GR::ObjectLink& grLink = *itLink;
		const GR::ObjectWrapper grSrc = grLink.src_end();
		argListType = "std::list<"+PE::getFullType(grSrc)+">"+REF;
		if(grObjects.find(grSrc)==grObjects.end())
			copyFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[grSrc])));
		const GR::ObjectWrapper grDst = grLink.dst_end();
		argListType = "std::list<"+PE::getFullType(grDst)+">"+REF;
		if(grObjects.find(grDst)==grObjects.end())
			copyFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[grDst])));
	}

	Function* udmcopyFunc = new Function;
	makeCallUdmcopyFun(udmcopyFunc, assocMapFunc, _groupLinks.empty());
	ruleDef.addPrivate( *udmcopyFunc);

	ruleDef.addPrivate( *copyFunc);
	FunctionCall* copyFuncCaller= new FunctionCall;
	copyFuncCaller->_func= copyFunc;
	copyFuncCaller->copyParamNames();
	_groupEffecDef->add( *copyFuncCaller);

	Definition* parentSet = new Definition;
	parentSet->_type = "std::set<Udm::Object>";
	parentSet->_variable = lowerBegin(PE::createUniqueName("parentSet"));
	copyFunc->add(*parentSet);
	for(set<GR::ObjectWrapper>::const_iterator itp=newParentObjs.begin();itp!=newParentObjs.end();++itp)
	{
		const GR::ObjectWrapper &pObj = *itp;
		std::string pObjType = PE::getFullType(pObj);
		Loop* plistLoop = new Loop;
		std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)pObj.name()));
		plistLoop->createIteratorLoop(itname,_boundObjList[pObj],"std::list<"+pObjType+">");
		//
		Definition* currpObj = new Definition;
		currpObj->_type = CST+SEP+pObjType+REF;
		currpObj->_variable = lowerBegin(PE::createUniqueName(CURR+(std::string)pObj.name()));
		currpObj->_initValue = PTR+itname;
		plistLoop->add(*currpObj);
		//check whether
		If* ifparent = new If;
		ifparent->createIfExprThenContinue(parentSet->_variable+".find("+currpObj->_variable+") != "+ parentSet->_variable+".end()");
		plistLoop->add(*ifparent);
		plistLoop->add(*new Expression(parentSet->_variable+".insert("+currpObj->_variable+")"));
		//define the UdmUtil::copy_assoc_map
		Definition* copy_map = new Definition;
		copy_map->_type = "UdmUtil::copy_assoc_map";
		copy_map->_variable = lowerBegin(PE::createUniqueName("copy_map"));
		plistLoop->add(*copy_map);
		//First copy the udm objects
		for(set<GR::ObjectWrapper>::const_iterator itobj=grObjects.begin();itobj!=grObjects.end();++itobj)
		{
			const GR::ObjectWrapper& cObj = *itobj;
			if(_groupAssocObjs.find(cObj)!=_groupAssocObjs.end()) continue;
			//Check whether its parent object is copied 
			if(isParentCopied(cObj, groupMembers)) continue;
			Loop* clistLoop = new Loop;
			makeUdmCopyCode(clistLoop, cObj,currpObj->_variable,copy_map->_variable,udmcopyFunc,plistLoop);
		}
		
		//Second copy the association class objects and make the connection
		for(set<GR::ObjectWrapper>::const_iterator itassoc=_groupAssocObjs.begin();itassoc!=_groupAssocObjs.end();++itassoc)
		{
			const GR::ObjectWrapper& assocObj = *itassoc;
			if(isParentCopied(assocObj, groupMembers)) continue;
			GR::ObjectWrapper srcCls, dstCls;
			if(!getAssociationClassEnd(assocObj, srcCls, dstCls)) continue;
			
			std::string fromVar, toVar;
			Loop* alistLoop = new Loop;
			makeUdmCopyCode(alistLoop, assocObj,currpObj->_variable,copy_map->_variable,udmcopyFunc,plistLoop,fromVar,toVar);

			std::string srcname, dstname;
			getAssociationRolenames(assocObj, srcname, dstname);
			
			//get the from ends
			Definition* fromSrc = new Definition;
			fromSrc->_type = CST+SEP+PE::getFullType(srcCls)+REF;
			fromSrc->_variable = lowerBegin(PE::createUniqueName("fromSrc"));
			fromSrc->_initValue = PE::getFullType(srcCls)+"::Cast("+fromVar+"."+srcname+"_end())";
			alistLoop->add(*fromSrc);
			Definition* fromDst = new Definition;
			fromDst->_type = CST+SEP+PE::getFullType(dstCls)+REF;
			fromDst->_variable = lowerBegin(PE::createUniqueName("fromDst"));
			fromDst->_initValue = PE::getFullType(dstCls)+"::Cast("+fromVar+"."+dstname+"_end())";
			alistLoop->add(*fromDst);

			std::string toSrc, toDst;
			makeGetCopiedAssociationEndsCode(alistLoop, copy_map->_variable, srcCls, dstCls, fromSrc->_variable, fromDst->_variable, toSrc, toDst);

			alistLoop->add(*new Expression(toVar+"."+srcname+"_end() = "+toSrc));
			alistLoop->add(*new Expression(toVar+"."+dstname+"_end() = "+toDst));
		}

		//Third, re-connect the simple association
		for(set<GR::ObjectLink>::const_iterator itlink=_groupLinks.begin();itlink!=_groupLinks.end();++itlink)
		{
			const GR::ObjectLink& grLink = *itlink;
			const GR::ObjectWrapper& grSrc = grLink.src_end();
			const GR::ObjectWrapper& grDst = grLink.dst_end();
			std::string srcListType = "std::list<"+PE::getFullType(grSrc)+">";
			std::string dstListType = "std::list<"+PE::getFullType(grDst)+">";
			
			Definition* itsrc = new Definition;
			itsrc->_type = srcListType+RES+"iterator";
			itsrc->_variable = lowerBegin(PE::createUniqueName(IT+(std::string)grSrc.name()));
			itsrc->_initValue = _boundObjList[grSrc]+".begin()";
			plistLoop->add(*itsrc);
			Definition* itdst = new Definition;
			itdst->_type = dstListType+RES+"iterator";
			itdst->_variable = lowerBegin(PE::createUniqueName(IT+(std::string)grDst.name()));
			itdst->_initValue = _boundObjList[grDst]+".begin()";
			plistLoop->add(*itdst);

			While* linkloop = new While;
			linkloop->_condition = itsrc->_variable+NEQ+_boundObjList[grSrc]+".end() && "+itdst->_variable+NEQ+_boundObjList[grDst]+".end()";
			Definition* srcObj = new Definition;
			srcObj->_type = CST+SEP+PE::getFullType(grSrc)+REF;
			srcObj->_variable = lowerBegin(PE::createUniqueName("srcObj"));
			srcObj->_initValue = PTR+itsrc->_variable;
			linkloop->add(*srcObj);
			Definition* dstObj = new Definition;
			dstObj->_type = CST+SEP+PE::getFullType(grDst)+REF;
			dstObj->_variable = lowerBegin(PE::createUniqueName("dstObj"));
			dstObj->_initValue = PTR+itdst->_variable;
			linkloop->add(*dstObj);
			
			std::string toSrc, toDst;
			makeGetCopiedAssociationEndsCode(linkloop, copy_map->_variable, grSrc, grDst, srcObj->_variable, dstObj->_variable, toSrc, toDst);

			bool isMulCardinality= ( 1 < 
			(unsigned int) getAssociationCardinality( ( std::string)grSrc.paradigmName(), (std::string)grSrc.nameSpace(), ( std::string)grSrc.classType(), ( std::string)grLink.dstRoleName(), false));
			Expression* newLink= new Expression;
			newLink->_expression= toSrc+ "."+ ( std::string)grLink.dstRoleName()+ "()"+(isMulCardinality ? PEQ : EQ)+ SEP+ toDst;
			linkloop->add( *newLink);

			linkloop->add(*new Expression(itsrc->_variable+INC));
			linkloop->add(*new Expression(itdst->_variable+INC));

			plistLoop->add(*linkloop);
		}

		copyFunc->add(*plistLoop);
	}
}

void GroupEffector::makeGetCopiedAssociationEndsCode(PE* currCode, const std::string& copy_map,
												 const GR::ObjectWrapper& grSrc, const GR::ObjectWrapper& grDst,
												 const std::string& fromSrc, const std::string& fromDst,
												 std::string& toSrc, std::string& toDst)
{
//Check the copy_map
	Definition* itSrc = new Definition;
	itSrc->_type = CST+SEP+"UdmUtil::copy_assoc_map::iterator"+REF;
	itSrc->_variable = lowerBegin(PE::createUniqueName("itsrc"));
	itSrc->_initValue = copy_map+".find("+fromSrc+")";
	currCode->add(*itSrc);
	Definition* itDst = new Definition;
	itDst->_type = CST+SEP+"UdmUtil::copy_assoc_map::iterator"+REF;
	itDst->_variable = lowerBegin(PE::createUniqueName("itdst"));
	itDst->_initValue = copy_map+".find("+fromDst+")";
	currCode->add(*itDst);
	If* ifend = new If;
	ifend->_expression = itSrc->_variable+"=="+copy_map+".end() || "+itDst->_variable+"=="+copy_map+".end()";
	ifend->add(*new Expression("throw udm_exception(\"Association Error:cannot find the source end or the destination end.\")"));//throw udm_exception(\"Association Error: cannot find the source end or the destination end.\"")));
	currCode->add(*ifend);

	//get the copied ends
	Definition* toSrcDef = new Definition;
	toSrcDef->_type = PE::getFullType(grSrc)+REF;
	toSrcDef->_variable = lowerBegin(PE::createUniqueName("toSrc"));
	toSrcDef->_initValue = PE::getFullType(grSrc)+"::Cast("+"("+PTR+itSrc->_variable+").second)";
	currCode->add(*toSrcDef);
	Definition* toDstDef = new Definition;
	toDstDef->_type = PE::getFullType(grDst)+REF;
	toDstDef->_variable = lowerBegin(PE::createUniqueName("toDst"));
	toDstDef->_initValue = PE::getFullType(grDst)+"::Cast("+"("+PTR+itDst->_variable+").second)";
	currCode->add(*toDstDef);

	toSrc = toSrcDef->_variable;
	toDst = toDstDef->_variable;
}

bool GroupEffector::isParentCopied(const GR::ObjectWrapper& grObj, const set<GR::GRBase>& groupmembers)
{
	set<GR::ObjectLink> links = grObj.dst();
	if(links.empty()) return false;
	for(set<GR::ObjectLink>::iterator i=links.begin();i!=links.end();++i)
	{
		const GR::ObjectLink &link = *i;
		if((string)link.relationshipType()=="Composition") 
		{
			const GR::ObjectWrapper &pObj = link.dst_end();
			if(groupmembers.find(pObj)!=groupmembers.end())
			{
				return true;
			}
			else return isParentCopied(pObj, groupmembers);
		}
	}		
}

void GroupEffector::makeUdmCopyCode(Loop* clistLoop, const GR::ObjectWrapper& fromObj, const std::string& toParent, 
									const std::string& copyMap, Function* udmcopyFunc, PE* currCode,
									std::string& fromVar, std::string& toVar)
{
	std::string cObjType = PE::getFullType(fromObj);
//	Loop* clistLoop = new Loop;
	std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)fromObj.name()));
	clistLoop->createIteratorLoop(itname,_boundObjList[fromObj],"std::list<"+cObjType+">");
	//
	Definition* currcObj = new Definition;
	currcObj->_type = cObjType;
	currcObj->_variable = lowerBegin(PE::createUniqueName(CURR+(std::string)fromObj.name()));
	currcObj->_initValue = PTR+itname;
	clistLoop->add(*currcObj);

	Definition* newcObj = new Definition;
	newcObj->_type = cObjType;
	newcObj->_variable = lowerBegin(PE::createUniqueName("new"+(std::string)fromObj.name()));
	clistLoop->add(*newcObj);

	FunctionCall* callUdmCopy= new FunctionCall;
	callUdmCopy->_func= udmcopyFunc;
	callUdmCopy->_params.push_back(currcObj->_variable);
	callUdmCopy->_params.push_back(newcObj->_variable);
	callUdmCopy->_params.push_back(toParent);
	callUdmCopy->_params.push_back(copyMap);
	clistLoop->add(*callUdmCopy);

	currCode->add(*clistLoop);	
	
	fromVar = currcObj->_variable;
	toVar = newcObj->_variable;
}

void GroupEffector::makeCallUdmcopyFun(Function* udmcopyFunc, Function* asscMapFunc, bool emptyLinks)
{
	udmcopyFunc->_name = "callUdmCopy";
	udmcopyFunc->_returnValue = "template <class T1, class T2 > bool";
	udmcopyFunc->insertInputParam(FuncParam_t( std::make_pair( "T1"+REF, "fromObj")));
	udmcopyFunc->insertInputParam(FuncParam_t( std::make_pair( "T1"+REF, "toObj")));
	udmcopyFunc->insertInputParam(FuncParam_t( std::make_pair( "T2"+REF, "toParent")));
	udmcopyFunc->insertInputParam(FuncParam_t( std::make_pair( "UdmUtil::copy_assoc_map"+REF, "copy_map")));
	//build the body of the function "callUdmCopy"
	udmcopyFunc->add(*new Expression("UdmUtil::copy_assoc_map::iterator currSelObjectIt = copy_map.find(fromObj)"));
	If* isSel = new If;
	isSel->_expression = ("currSelObjectIt != copy_map.end()");
	isSel->add(*new Expression("Udm::Object& a_toObj = (*currSelObjectIt).second"));
	isSel->add(*new Expression("Udm::Object& a_toParent = a_toObj.GetParent()"));
	If* isSame = new If;
	isSame->_expression = "a_toObj != fromObj || a_toParent == toParent";
	isSame->add(*new Expression("return false"));
	isSel->add(*isSame);
	udmcopyFunc->add(*isSel);
	udmcopyFunc->add(*new Expression("UdmUtil::copy_assoc_map curr_copy_map"));
	If* isInstSubtype = new If;
	isInstSubtype->_expression = ("fromObj.isInstance() || fromObj.isSubtype()");
	isInstSubtype->add(*new Expression("T1 fromObj_arch = T1::Cast(fromObj.archetype())"));
	If* isInst = new If;
	isInst->_expression = ("fromObj.isInstance()");
	isInst->add(*new Expression("toObj = fromObj_arch.CreateInstance(toParent)"));
	isInst->addElse(*new Expression("toObj = fromObj_arch.CreateDerived(toParent)"));
	isInstSubtype->add(*isInst);
	FunctionCall* callAssocMap= new FunctionCall;
	callAssocMap->_func= asscMapFunc;
	callAssocMap->_params.push_back("fromObj");;
	callAssocMap->_params.push_back("toObj");
	callAssocMap->_params.push_back("curr_copy_map");
	isInstSubtype->add( *callAssocMap);
	isInstSubtype->addElse(*new Expression("toObj= T1::Create(toParent)"));
	isInstSubtype->addElse(*new Expression("UdmUtil::CopyObjectHierarchy(fromObj.__impl(), toObj.__impl(), toObj.__impl()->__getdn(),curr_copy_map)"));
	isInstSubtype->addElse(*new Expression("curr_copy_map[fromObj] = toObj"));

	udmcopyFunc->add(*isInstSubtype);
	if(emptyLinks)
		udmcopyFunc->add(*new Expression("copy_map.insert(curr_copy_map.begin(), curr_copy_map.end())"));
	else
	{
		Loop* currMapLoop = new Loop;
		currMapLoop->createIteratorLoop("currMapIt","curr_copy_map","UdmUtil::copy_assoc_map::iterator");
		If* iffind = new If;
		iffind->_expression = "copy_map.find((*currMapIt).first)==copy_map.end() || (*currMapIt).first!=(*currMapIt).second";
		iffind->add(*new Expression("copy_map[(*currMapIt).first] = (*currMapIt).second"));
		currMapLoop->add(*iffind);
		udmcopyFunc->add(*currMapLoop);
	}
	udmcopyFunc->add(*new Expression("return true"));
}

void GroupEffector::makeUdmcopyAssocMapFunc(Function* assocMapFunc)
{
	assocMapFunc->_name = "makeUdmcopyAssocMap";
	assocMapFunc->insertInputParam(FuncParam_t( std::make_pair( CST+SEP+"Udm::Object"+REF, "fromObj")));
	assocMapFunc->insertInputParam(FuncParam_t( std::make_pair( CST+SEP+"Udm::Object"+REF, "toObj")));
	assocMapFunc->insertInputParam(FuncParam_t( std::make_pair( "UdmUtil::copy_assoc_map"+REF, "copy_map")));
	//build the body of the function "makeUdmcopyAssocMap"
	assocMapFunc->add(*new Expression("copy_map[fromObj] = toObj"));
	assocMapFunc->add(*new Expression("toObj.__impl()->CopyAttributesFrom(fromObj.__impl())"));
	
	Definition* fromChildren = new Definition;
	fromChildren->_type = "set<Udm::Object>";
	fromChildren->_variable = "from_children";
	fromChildren->_initValue = "fromObj.GetChildObjects()";
	assocMapFunc->add(*fromChildren);
	Definition* toChildren = new Definition;
	toChildren->_type = "set<Udm::Object>";
	toChildren->_variable = "to_children";
	toChildren->_initValue = "toObj.GetChildObjects()";
	assocMapFunc->add(*toChildren);

	Loop* fromloop = new Loop;
	fromloop->createIteratorLoop("itfrom",fromChildren->_variable,fromChildren->_type);
	Definition* currFrom = new Definition;
	currFrom->_type = CST+SEP+"Udm::Object"+REF;
	currFrom->_variable = "from_child";
	currFrom->_initValue = PTR+"itfrom";
	fromloop->add(*currFrom);
	Loop* toloop = new Loop;
	toloop->createIteratorLoop("itto", toChildren->_variable, toChildren->_type);
	Definition* currTo = new Definition;
	currTo->_type = CST+SEP+"Udm::Object"+REF;
	currTo->_variable = "to_child";
	currTo->_initValue = PTR+"itto";
	toloop->add(*currTo);
	If* sameArch = new If;
	sameArch->_expression = currFrom->_variable+".archetype() == "+currTo->_variable+".archetype()";
	sameArch->add(*new Expression("makeUdmcopyAssocMap("+currFrom->_variable+", "+currTo->_variable+", copy_map)"));
	sameArch->add(*new Expression(toChildren->_variable+".erase("+currTo->_variable+")"));
	sameArch->add(*new Expression("break"));
	toloop->add(*sameArch);
	fromloop->add(*toloop);
	assocMapFunc->add(*fromloop);
}

void GroupEffector::deleteGroup(Class& ruleDef, set<GR::GRBase>& groupMembers)
{
	Function* deleGroupFunc = new Function;
	deleGroupFunc->_name = "deleteGroup";

	set<GR::ObjectWrapper> delObjs = _groupObjs;
	delObjs.insert(_groupAssocObjs.begin(), _groupAssocObjs.end());
	delObjs.insert(_groupSetObjs.begin(), _groupSetObjs.end());

	makeDeleteFunc(deleGroupFunc, delObjs, _groupLinks);
	
	ruleDef.addPrivate( *deleGroupFunc);
	FunctionCall* delFuncCaller= new FunctionCall;
	delFuncCaller->_func= deleGroupFunc;
	delFuncCaller->copyParamNames();
	_groupEffecDef->add( *delFuncCaller);
}

void GroupEffector::makeDeleteFunc(Function* delFunc, set<GR::ObjectWrapper>& delObjs, set<GR::ObjectLink>& delLinks)
{
	std::string argListType;
	set<GR::ObjectWrapper> delObjArgs;
	for(set<GR::ObjectLink>::const_iterator itLink=delLinks.begin();itLink!=delLinks.end();++itLink)
	{
		const GR::ObjectLink& delLink = *itLink;
		const GR::ObjectWrapper& delSrc = delLink.src_end();
		argListType = "std::list<"+PE::getFullType(delSrc)+">"+REF;
		if(delObjArgs.find(delSrc)==delObjArgs.end())
		{
			delFunc->insertInputParam(FuncParam_t( std::make_pair(argListType, _boundObjList[delSrc])));
			delObjArgs.insert(delSrc);
		}
		const GR::ObjectWrapper& delDst = delLink.dst_end();
		argListType = "std::list<"+PE::getFullType(delDst)+">"+REF;
		if(delObjArgs.find(delDst)==delObjArgs.end())
		{
			delFunc->insertInputParam(FuncParam_t( std::make_pair(argListType, _boundObjList[delDst])));
			delObjArgs.insert(delDst);
		}

		std::string srcListType = "std::list<"+PE::getFullType(delSrc)+">";
		std::string dstListType = "std::list<"+PE::getFullType(delDst)+">";
		
		Definition* itsrc = new Definition;
		itsrc->_type = srcListType+RES+"iterator";
		itsrc->_variable = lowerBegin(PE::createUniqueName(IT+(std::string)delSrc.name()));
		itsrc->_initValue = _boundObjList[delSrc]+".begin()";
		delFunc->add(*itsrc);
		Definition* itdst = new Definition;
		itdst->_type = dstListType+RES+"iterator";
		itdst->_variable = lowerBegin(PE::createUniqueName(IT+(std::string)delDst.name()));
		itdst->_initValue = _boundObjList[delDst]+".begin()";
		delFunc->add(*itdst);

		While* whileloop = new While;
		whileloop->_condition = itsrc->_variable+NEQ+_boundObjList[delSrc]+".end() && "+itdst->_variable+NEQ+_boundObjList[delDst]+".end()";
		Definition* srcObj = new Definition;
		srcObj->_type = PE::getFullType(delSrc)+REF;
		srcObj->_variable = lowerBegin(PE::createUniqueName("srcObj"));
		srcObj->_initValue = PTR+itsrc->_variable;
		whileloop->add(*srcObj);
		Definition* dstObj = new Definition;
		dstObj->_type = PE::getFullType(delDst)+REF;
		dstObj->_variable = lowerBegin(PE::createUniqueName("dstObj"));
		dstObj->_initValue = PTR+itdst->_variable;
		whileloop->add(*dstObj);
		
		If* ifneithernull = new If;
		ifneithernull->_expression = srcObj->_variable+" && "+dstObj->_variable;
		ifneithernull->add(*new Expression("const Uml::Class clsNull(NULL)"));
		ifneithernull->add(*new Expression("Udm::Object::AssociationInfo assoInfo(clsNull)"));
		ifneithernull->add(*new Expression("assoInfo.strSrcRoleName = \""+(std::string)delLink.srcRoleName()+"\""));
		ifneithernull->add(*new Expression("assoInfo.strDstRoleName = \""+(std::string)delLink.dstRoleName()+"\""));
		ifneithernull->add(*new Expression(srcObj->_variable+".DeleteLink("+dstObj->_variable+", assoInfo)"));
		whileloop->add(*ifneithernull);
		
		whileloop->add(*new Expression(itsrc->_variable+INC));
		whileloop->add(*new Expression(itdst->_variable+INC));
		
		delFunc->add(*whileloop);
	}

	for(set<GR::ObjectWrapper>::const_iterator itObj=delObjs.begin();itObj!=delObjs.end();++itObj)
	{
		const GR::ObjectWrapper& delObj = *itObj;
		//Check whether the parent object belongs to the to-be-deleted group
		bool delParent = false;
		set<GR::ObjectLink> links=delObj.dst();
		for(set<GR::ObjectLink>::iterator i=links.begin();i!=links.end();++i)
		{
			const GR::ObjectLink &link = *i;
			if((string)link.relationshipType()=="Composition") 
			{
				const GR::ObjectWrapper &pObj = link.dst_end();
				if(delObjs.find(pObj)!=delObjs.end())
				{
					delParent = true;
					break;
				}
			}
		}		
		if(delParent) continue;

		argListType = "std::list<"+PE::getFullType(delObj)+">"+REF;
		if(delObjArgs.find(delObj)==delObjArgs.end())
		{
			delFunc->insertInputParam(FuncParam_t( std::make_pair( argListType, _boundObjList[delObj])));
			delObjArgs.insert(delObj);
		}

		std::string objListType = "std::list<"+PE::getFullType(delObj)+">";
		std::string itname = lowerBegin(PE::createUniqueName(IT+(std::string)delObj.name()));
		Loop* objLoop = new Loop;
		objLoop->createIteratorLoop(itname,_boundObjList[delObj],objListType);
		Definition* currObj = new Definition;
		currObj->_type = PE::getFullType(delObj);
		currObj->_variable = lowerBegin(PE::createUniqueName((std::string)delObj.name()));
		currObj->_initValue = PTR+itname;
		objLoop->add(*currObj);
		If* isnull = new If;
		isnull->_expression =currObj->_variable;		
		isnull->add(*new Expression(currObj->_variable+".DeleteObject()"));
		objLoop->add(*isnull);
		delFunc->add(*objLoop);
	}
}

void GroupEffector::postDelete(Class& ruleDef)
{
	if(_deleteObjs.empty() && _postDeleteLinks.empty()) return;
	
	Function* deleteFunc = new Function;
	deleteFunc->_name = "postDelete";

	makeDeleteFunc(deleteFunc, _deleteObjs, _postDeleteLinks);
	
	ruleDef.addPrivate( *deleteFunc);
	FunctionCall* delFuncCaller= new FunctionCall;
	delFuncCaller->_func= deleteFunc;
	delFuncCaller->copyParamNames();
	_groupEffecDef->add( *delFuncCaller);
}

void GroupEffector::createRTT()
{
	set< GR::ObjectLink> links= _rule.ObjectLink_kind_children();
	for( set< GR::ObjectLink>::const_iterator itLinks= links.begin(); itLinks!= links.end(); ++itLinks)
	{
		const GR::ObjectLink& currlink= *itLinks;
		if (( std::string)currlink.relationshipType() != "TracesTo") continue;

		GR::ObjectWrapper srcOW = currlink.src_end();
		GR::ObjectWrapper dstOW = currlink.dst_end();
		if( srcOW==Udm::null || dstOW==Udm::null)
			throw CGEx("FATAL: Invalid TracesTo relationship found.");
		
		std::string slistName = _boundObjList[srcOW];
		std::string dlistName = _boundObjList[dstOW];
		std::string src_itname = lowerBegin(PE::createUniqueName(IT+(std::string)srcOW.name()));
		std::string dst_itname = lowerBegin(PE::createUniqueName(IT+(std::string)dstOW.name()));

		Loop* slistloop = new Loop;
		slistloop->createIteratorLoop(src_itname, slistName, "std::list<"+PE::getFullType(srcOW)+">");
		Loop* dlistloop = new Loop;
		dlistloop->createIteratorLoop(dst_itname, dlistName, "std::list<"+PE::getFullType(dstOW)+">");
		dlistloop->add(*new Expression("RTTGenerator::Instance()->generateRTT((*"+src_itname+").uniqueId(), (*"+src_itname+").__impl()->__getdn()->uniqueId(), (std::string)(*"+src_itname+").name(), (*"+
			dst_itname+").uniqueId(), (*"+dst_itname+").__impl()->__getdn()->uniqueId(), (std::string)(*"+dst_itname+").name())"));	
		slistloop->add(*dlistloop);
		_groupEffecDef->add(*slistloop);
	}
}