/* NAME:	Effector
 * AUTHOR:	Attila Vizhanyo - viza@isis-server.isis.vanderbilt.edu
 *
 * Generate code, which executes actions 'create new' and 'delete' on 
 * 'GR::PatternObject'-s and 'GR::ObjectLink'-s.
 */
#ifndef __GROUPEFFECTOR_H
#define	__GORUPEFFECTOR_H

#ifdef _MSC_VER
#pragma warning (disable : 4786) 
#pragma warning (disable : 4503)
#endif

#include "Effector.h"

class GroupEffector : public Effector
{
public:
	GroupEffector( const GRSimpleRule& rule, 
		const GeneralBackend& backend, 
		std::map< GR::ObjectWrapper, std::string>& boundObjects,
		ICIntCompiler& cc,
		const ParamDatas_t& ruleOutputs);
	
	void makeGroupRule( Class& ruleDef);
	// effector for the rule with Group element
	void executeRightSide( PEComposite& code);
	void executeGroup( Class& ruleDef);

protected:
	void createOutputObject(const GR::ObjectWrapper& parentOW, GR::ObjectWrapper& childOW, const std::string& childRole);
	void bindRemainingObjectLinks();
	void createNewObjectLink(const GR::ObjectLink& linkToCreate);
	void callAttributeMapping( Class& ruleDef);
	void preDeleteObjectLink(Class& ruleDef);
	void executeGroupAction(Class& ruleDef);
	void postDelete(Class& ruleDef);
	void moveGroup(Class& ruleDef, set<GR::GRBase>& groupMembers);
	void copyGroup(Class& ruleDef, set<GR::GRBase>& groupMembers);
	void deleteGroup(Class& ruleDef, set<GR::GRBase>& groupMembers);
	bool getAssociationClassEnd(const GR::ObjectWrapper& assocCls, GR::ObjectWrapper& srcCls, GR::ObjectWrapper& dstCls);
	bool getAssociationLinks(const GR::ObjectWrapper& assocCls, GR::ObjectLink& srcLink, GR::ObjectLink& dstLink);
	bool getAssociationRolenames(const GR::ObjectWrapper& assocCls, std::string& srcRolename, std::string& dstRolename);
//	std::string getPatternObjectType(const GR::ObjectWrapper& grobj);
	void createUdmCopy(PE* currCode, std::string fromObj, std::string toObj, bool copyAttribute=false);
	void makeUdmCopyCode(Loop* clistLoop, const GR::ObjectWrapper& fromObj, const std::string& toParent, 
						const std::string& copyMap, Function* udmcopyFunc, PE* currCode,
						std::string& fromVar=std::string(""), std::string& toVar=std::string(""));
	void makeCallUdmcopyFun(Function* udmcopyFunc, Function* asscMapFunc, bool emptyLinks=true);
	void makeUdmcopyAssocMapFunc(Function* assocMapFunc);
	void makeDeleteFunc(Function* delFunc, set<GR::ObjectWrapper>& delObjs, set<GR::ObjectLink>& delLinks);
	void makeGetCopiedAssociationEndsCode(PE* currCode, const std::string& copy_map,
									const GR::ObjectWrapper& grSrc, const GR::ObjectWrapper& grDst,
									const std::string& fromSrc, const std::string& fromDst,
									std::string& toSrc, std::string& toDst);
	bool isParentCopied(const GR::ObjectWrapper& grObj, const set<GR::GRBase>& groupmembers);
		// Create code for RTT
	void createRTT();

protected:
	Function* _groupEffecDef;
	GR::PatternGroup _group;
	map<GR::ObjectWrapper,std::string > _boundObjList;
	ParamDatas_t _ruleOutputs;
	set<GR::ObjectWrapper> _groupObjs;
	set<GR::ObjectWrapper> _groupSetObjs;
	set<GR::ObjectWrapper> _groupAssocObjs;
	set<GR::ObjectLink> _groupLinks;
	set<GR::ObjectLink> _newLinks;
	set<GR::ObjectWrapper> _deleteObjs;
	set<GR::ObjectLink> _preDeleteLinks;
	set<GR::ObjectLink> _postDeleteLinks;
};
#endif //__GROUPEFFECTOR_H