#include "CopyToMemDNCodePrinter.h"
#include "../CodeGenerator/PEStringTable.h"
#include "GReATUtils.h"

std::string CopyToMemDNCodePrinter::_dnsVecName= "";
std::string CopyToMemDNCodePrinter::_udmProjectName= "";

void CopyToMemDNCodePrinter::generateCreateMemDN( const std::string& hostFileName, const std::string& meta, const std::string& paradigm, const std::string& root, const std::string& dtd)
{
	// Udm::SmartDataNetwork mem_<meta>( <meta>::diagram);
	_memDNDef= constructObj( _memDNDef);
	_memDNDef->_type= "Udm::SmartDataNetwork";
	_memDNDef->_variable= PE::createUniqueName( "mem_"+ meta);
	_memDNDef->addArg( meta+ RES+ "diagram");
	// mem_<meta>.CreateNew( <meta>_id.mem, UseXsd()( <hostFileName> ? <dtd> : <meta>, <meta>::<root>::meta, "Udm::CHANGES_PERSIST_ALWAYS");
	_memDNCreateExpr= constructObj( _memDNCreateExpr);
	std::string metalocator= "UseXSD()("+ SEP+ hostFileName+ ") ? "+ dtd+ " : \"" + paradigm + "\"";
	_memDNCreateExpr->_expression= _memDNDef->_variable+ ".CreateNew(" + SEP+ "\""+ PE::createUniqueName( meta)+ ".mem\","+ SEP+
		metalocator+ ","+  SEP+ meta+ RES+ root+ RES+ "meta,"+ SEP+ "Udm::CHANGES_PERSIST_ALWAYS" + ")";

	//added by feng for RTT 01/05/10
	_memRTTExpr= constructObj( _memRTTExpr);
	_memRTTExpr->_expression= "RTTGenerator::Instance()->generateDataNetwork("+_memDNDef->_variable+".testdn()->uniqueId(), getFullPath((std::string)cwd,"+hostFileName+"), dropPath("+hostFileName+"))";
}

void CopyToMemDNCodePrinter::generateCopyMemDN( const std::string& lhs, const std::string& rhs, const bool& reverse)
{
	// lhs= rhs
	srcdn = rhs;
	dstdn = lhs;
	rtt_reverse = reverse;
	_memCopyExpr= constructObj( _memCopyExpr);
	//_memCopyExpr->_expression= lhs+ EQ+ SEP+ rhs;
	_memCopyExpr->_expression="UdmUtil::CopyObjectHierarchy("+rhs+".GetRootObject().__impl(), "+lhs+".GetRootObject().__impl(), "+lhs+".testdn(),udm_copyMap)";
}

void CopyToMemDNCodePrinter::addCreateMemCode( PEComposite& addTo) const
{	
	if ( _memDNDef) addTo.add( *_memDNDef);
	if ( _memDNCreateExpr) addTo.add( *_memDNCreateExpr);
	if( _memRTTExpr) addTo.add(*_memRTTExpr);
}

void CopyToMemDNCodePrinter::addCopyMemCode( PEComposite& addTo) const
{
	if ( _memCopyExpr) 
	{
		addTo.add(*new Expression("udm_copyMap.clear()"));
		addTo.add(*new Expression("rtt_copyIdMap.clear()"));
		addTo.add( *_memCopyExpr);
		if(rtt_reverse)
		{
			addTo.add(*new Expression("RTTGenerator::convertUdmCopyMap(udm_copyMap, rtt_copyIdMap, true)"));
			addTo.add(*new Expression("RTTGenerator::Instance()->addUdmCopyMap("+dstdn+".testdn()->uniqueId(), rtt_copyIdMap)"));
		}
		else
		{
			addTo.add(*new Expression("RTTGenerator::convertUdmCopyMap(udm_copyMap, rtt_copyIdMap, false)"));
			addTo.add(*new Expression("RTTGenerator::Instance()->addUdmCopyMap("+srcdn+".testdn()->uniqueId(), rtt_copyIdMap)"));
		}
	}

}

void ReadCopyToMemDNCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
{
	//
	ReadDNProjectCodePrinter::generateOpenDN( hostFileName, meta, paradigm, dtd, mode);
	if(ReadDNProjectCodePrinter::_hostRTTExpr)
		ReadDNProjectCodePrinter::_hostRTTExpr = 0;
	//
	generateCreateMemDN( hostFileName, meta, paradigm, root, dtd);
	generateCopyMemDN( getMemFileName(), ReadDNProjectCodePrinter::getSdnFileName(), true);
	//
	std::string dnSpecifier= "\""+ PE::createUniqueName( meta)+ ".mem\"";
	ReadDNProjectCodePrinter::generateSDNS( getMemFileName(), dnSpecifier, meta, dnsvecname);
	ReadDNProjectCodePrinter::generateDNFromProject( dnSpecifier, meta, udmPrj);
}

void ReadWriteCopyToMemDNCodePrinter::generateCopyBackDN()
{
	_copyBackExpr= constructObj( _copyBackExpr);
//	_copyBackExpr->_expression= ReadDNProjectCodePrinter::getSdnFileName()+ EQ+ SEP+ getMemFileName();
	std::string lhs = ReadDNProjectCodePrinter::getSdnFileName();
	std::string rhs = getMemFileName();
	_copyBackExpr->_expression= "UdmUtil::CopyObjectHierarchy("+rhs+".GetRootObject().__impl(), "+lhs+".GetRootObject().__impl(), "+lhs+".testdn(),udm_copyMap)";
}

void ReadWriteCopyToMemDNCodePrinter::addCopyBackDN( PEComposite& addTo) const 
{
	if ( _copyBackExpr) 
	{
		addTo.add(*new Expression("udm_copyMap.clear()"));
		addTo.add(*new Expression("rtt_copyIdMap.clear()"));
		addTo.add(*new Expression("vector<Udm::ObjectImpl*> vec = "+_hostDef->_variable+".GetRootObject().__impl()->getChildren(NULL, NULL)"));
		Loop* vecLoop= new Loop;
		vecLoop->createIteratorLoop( "vec_i", "vec", "vector<Udm::ObjectImpl*>");
		vecLoop->add(*new Expression("(*vec_i)->detach()"));
		vecLoop->add(*new Expression("(*vec_i)->release()"));
		addTo.add(*vecLoop);
		addTo.add( *_copyBackExpr);
		addTo.add(*new Expression("RTTGenerator::convertUdmCopyMap(udm_copyMap, rtt_copyIdMap, false)"));
		addTo.add(*new Expression("RTTGenerator::Instance()->addUdmCopyMap("+getMemFileName()+".testdn()->uniqueId(), rtt_copyIdMap)"));
		//close memory backend
		addTo.add(*new Expression(getMemFileName()+".CloseNoUpdate()"));
	}
}

void ReadWriteCopyToMemDNCodePrinter::getOpenCode( PEComposite& addTo) const
{
	ReadDNProjectCodePrinter::addOpenDNCode( addTo);
	addCreateMemCode( addTo);
	addCopyMemCode( addTo);
	ReadDNProjectCodePrinter::addSDNSCode( addTo);
}

void ReadWriteCopyToMemDNCodePrinter::addCopyMemCode( PEComposite& addTo) const
{
	if ( _memCopyExpr) 
	{
		addTo.add(*new Expression("udm_copyMap.clear()"));
		addTo.add(*new Expression("rtt_copyIdMap.clear()"));
		addTo.add( *_memCopyExpr);
	}
}

void WriteCopyToMemDNCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
{
	//
	generateCreateMemDN( hostFileName, meta, paradigm, root, dtd);
	//
	std::string dnSpecifier= "\""+ PE::createUniqueName( meta)+ ".mem\"";
	WriteDNProjectCodePrinter::generateSDNS( getMemFileName(), dnSpecifier, meta, dnsvecname);
	WriteDNProjectCodePrinter::generateDNFromProject( dnSpecifier, meta, udmPrj);
	//
	WriteDNProjectCodePrinter::generateOpenDN( hostFileName, meta, paradigm, root, dtd, mode);
	
	//WriteDNProjectCodePrinter::_hostRTTExpr= constructObj( WriteDNProjectCodePrinter::_hostRTTExpr);
	if(WriteDNProjectCodePrinter::_hostRTTExpr)
		WriteDNProjectCodePrinter::_hostRTTExpr = 0;
}

void WriteCopyCopyToMemDNCodePrinter::generateSaveAsDN()
{
	_saveAsExpr= constructObj( _saveAsExpr);
	_saveAsExpr->_expression= ReadDNProjectCodePrinter::getSdnFileName()+ ".SaveAs(" + SEP+ _copyName+ ")";
}

void WriteCopyCopyToMemDNCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) 
{
	ReadCopyToMemDNCodePrinter::generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, udmPrj, dnsvecname, mode);
	_copyName= copyFileName;
	_memRTTExpr= constructObj( _memRTTExpr);
	_memRTTExpr->_expression= "RTTGenerator::Instance()->generateDataNetwork("+_memDNDef->_variable+".testdn()->uniqueId(), getFullPath((std::string)cwd,"+copyFileName+"), dropPath("+copyFileName+"))";
}
