/* NAME:	DN Code Printer
 * AUTHOR:	Attila Vizhanyo - viza@isis.vanderbilt.edu
 *
 * Abstract interface for Code Printer classes.
 */
#ifndef DNCODEPRINTER_H
#define DNCODEPRINTER_H

#include <string>
#include "../CodeGenerator/PE.h"

// This class is a replicated base class in a multiple inheritance chain,
// thus it is important that this class is abstract.
// DO NOT add data members to this class!!
class DNCodePrinter
{
public:
	// Generate (and store for subsequent use) init, open & close code 
	// based on the arguments: 'hostFileName', 'copyFileName', 'meta', 'root', 'dtd'.
	virtual void generate( 
			const std::string& hostFileName,
			const std::string& copyFileName,
			const std::string& meta, 
			const std::string& paradigm, 
			const std::string& root,
			const std::string& dtd
		) = 0;
	// Add the previously generated open, close, get-Dn code to 'addTo'
	virtual void getOpenCode( PEComposite& addTo) const = 0;
	virtual void getCloseCode( PEComposite& addTo) const = 0;
	virtual void getDnCode( PEComposite& addTo) const = 0;
	// Returns with the generated dn name.
	virtual const std::string& getDN() const = 0;
};

#endif //DNCODEPRINTER_H