#include "DNProjectCodePrinter.h"
#include "../Codegenerator/PEStringTable.h"
#include "GReATUtils.h"

std::string DNProjectCodePrinter::_udmProjectName;
std::string DNProjectCodePrinter::_dnsVecName;

///////////////////////////////////////////////////////////////////////////////
void DNProjectCodePrinter::generateSDNS(
			const std::string& hostDN,										
			const std::string& hostFileName,
			const std::string& meta, 
			const std::string& dnsvecname
			)
{
	// Udm::StaticDataNetworkSpecifier sdns_<meta>( <hostFileName>, &<hostDN>);
	_udmSDNSDef= constructObj( _udmSDNSDef);
	_udmSDNSDef->_type= "Udm::StaticDataNetworkSpecifier";
	_udmSDNSDef->_variable= PE::createUniqueName( "sdns_"+ meta);
	_udmSDNSDef->addArg( hostFileName);
	_udmSDNSDef->addArg( REF+ hostDN);
	// <dnsvecname>.push_back( sdns_<meta>);
	_pushVecExpr= constructObj( _pushVecExpr);
	_pushVecExpr->_expression= dnsvecname+ ".push_back("+ SEP+ lowerBegin( _udmSDNSDef->_variable)+ ")";
}

void DNProjectCodePrinter::generateDNFromProject(
			const std::string& dnSpecifier,
			const std::string& meta,
			const std::string& udmPrj
			 )
{
	// Udm::DataNetwork& <meta>_ref= <udmPrj>.GetDataNetwork( <dnSpecifier>);
	_getDNFromProjectDef= constructObj( _getDNFromProjectDef);
	_getDNFromProjectDef->_type= "Udm::DataNetwork&";
	_getDNFromProjectDef->_variable= PE::createUniqueName( meta+ "_ref");
	_getDNFromProjectDef->_initValue= udmPrj+ ".GetDataNetwork("+ SEP+ dnSpecifier+ ")";
}

///////////////////////////////////////////////////////////////////////////////
void ReadDNProjectCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,			// unused
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		)
{
	// Udm::SmartDataNetwork sdn_<meta>( <meta>::diagram);
	// sdn_<meta>.OpenExisting( <hostFileName>, UseXsd()( <hostFileName> ? <dtd> : <paradigm>, <mode>);
	generateOpenDN( hostFileName, meta, paradigm, dtd, mode);
	// Udm::StaticDataNetworkSpecifier sdns_<meta>( <hostFileName>, &<_hostDef->_variable>);
	// <dnsVecName>.push_back( sdns_<meta>);
	generateSDNS( _hostDef->_variable, hostFileName, meta, dnsvecname);
	// Udm::DataNetwork& <meta>_ref= <udmPrj>.GetDataNetwork( <hostFileName>);
	generateDNFromProject( hostFileName, meta, udmPrj);
}

void ReadDNProjectCodePrinter::generateOpenDN(
		const std::string& hostFileName,
		const std::string& meta,
		const std::string& paradigm,
		const std::string& dtd,
		const std::string& mode
		)
{
	// Udm::SmartDataNetwork sdn_<meta>( <meta>::diagram);
	_hostDef= constructObj( _hostDef);
	_hostDef->_type= "Udm::SmartDataNetwork";
	_hostDef->_variable= PE::createUniqueName( "sdn"+ meta);
	_hostDef->addArg( meta+ RES+ "diagram");
	// sdn_<meta>.OpenExisting( <hostFileName>, UseXsd()( <hostFileName> ? <dtd> : <paradigm>, <mode>);
	_hostOpenExpr= constructObj( _hostOpenExpr);
	std::string metalocator= "UseXSD()("+ SEP+ hostFileName+ ") ? "+ dtd+ " : \"" + paradigm + "\"";
	_hostOpenExpr->_expression= _hostDef->_variable+ ".OpenExisting(" + SEP+ hostFileName+ ","+ SEP+
		metalocator+ ","+ SEP+ mode + ")" ;

	//added by feng for RTT 01/05/10
	_hostRTTExpr= constructObj( _hostRTTExpr);
	_hostRTTExpr->_expression= "RTTGenerator::Instance()->generateDataNetwork("+_hostDef->_variable+".testdn()->uniqueId(), getFullPath((std::string)cwd,"+hostFileName+"), dropPath("+hostFileName+"))";
}

void ReadDNProjectCodePrinter::generateClose() 
{
	generateCloseDN( false);
}

void ReadDNProjectCodePrinter::generateCloseDN( bool closeWithUpdate) 
{
	// <_hostDef->_variable>.CloseNoUpdate();
	_hostCloseExpr= constructObj( _hostCloseExpr);
	_hostCloseExpr->_expression= lowerBegin( getSdnFileName()) + ( closeWithUpdate ? ".CloseWithUpdate()" : ".CloseNoUpdate()");
}

///////////////////////////////////////////////////////////////////////////////
void WriteDNProjectCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		)
{
	generateOpenDN( hostFileName, meta, paradigm, root, dtd, mode);
	// Udm::StaticDataNetworkSpecifier sdns_<meta>( <hostFileName>, &<_hostDef->_variable>);
	// <dnsVecName>.push_back( sdns_<meta>);
	generateSDNS( _hostDef->_variable, hostFileName, meta, dnsvecname);
	// Udm::DataNetwork& <meta>_ref= <udmPrj>.GetDataNetwork( <hostFileName>);
	generateDNFromProject( hostFileName, meta, udmPrj);
}

void WriteDNProjectCodePrinter::generateOpenDN(
		const std::string& hostFileName,
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& mode
		)
{
	// Udm::SmartDataNetwork sdn_<meta>( <meta>::diagram);
	_hostDef= constructObj( _hostDef);
	_hostDef->_type= "Udm::SmartDataNetwork";
	_hostDef->_variable= PE::createUniqueName( "sdn"+ meta);
	_hostDef->addArg( meta+ RES+ "diagram");
	// sdn_<meta>.CreateNew( <hostFileName>, UseXsd()( <hostFileName> ? <dtd> : <paradigm>, <meta>::<root>::meta, mode);
	_hostCreateExpr= constructObj( _hostCreateExpr);
	std::string metalocator= "UseXSD()("+ SEP+ hostFileName+ ") ? "+ dtd+ " : \"" + paradigm + "\"";
	_hostCreateExpr->_expression= _hostDef->_variable+ ".CreateNew(" + SEP+ hostFileName+ ","+ SEP+
		metalocator+ ","+  SEP+ meta+ RES+ root+ RES+ "meta,"+ SEP+ mode + ")";

	//added by feng for RTT 01/05/10
	_hostRTTExpr= constructObj( _hostRTTExpr);
	_hostRTTExpr->_expression= "RTTGenerator::Instance()->generateDataNetwork("+_hostDef->_variable+".testdn()->uniqueId(), getFullPath((std::string)cwd,"+hostFileName+"), dropPath("+hostFileName+"))";
}

void WriteDNProjectCodePrinter::generateClose()
{
	// <_hostDef->_variable>.CloseWithUpdate();
	_hostCloseExpr= constructObj( _hostCloseExpr);
	_hostCloseExpr->_expression= lowerBegin( getSdnFileName()) + ".CloseWithUpdate()";
}

///////////////////////////////////////////////////////////////////////////////
void ReadWriteDNProjectCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		)
{
	ReadDNProjectCodePrinter::generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, udmPrj, dnsvecname, mode);
}

void ReadWriteDNProjectCodePrinter::generateClose()
{
	//  <_hostDef->_variable>.CloseWithUpdate();
	ReadDNProjectCodePrinter::generateCloseDN( true);
}

///////////////////////////////////////////////////////////////////////////////
void WriteCopyDNProjectCodePrinter::generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,
		const std::string& meta,
		const std::string& paradigm,
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		)
{
	_copyName= copyFileName;
	ReadDNProjectCodePrinter::generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, udmPrj, dnsvecname, mode);
	_hostRTTExpr= constructObj( _hostRTTExpr);
	_hostRTTExpr->_expression= "RTTGenerator::Instance()->generateDataNetwork("+_hostDef->_variable+".testdn()->uniqueId(), getFullPath((std::string)cwd,"+_copyName+"), dropPath("+_copyName+"))";
}

void WriteCopyDNProjectCodePrinter::generateClose()
{	
	// <_hostDef->_variable>.SaveAs( <_copyName>);
	_hostSaveAsExpr= constructObj( _hostSaveAsExpr);
	_hostSaveAsExpr->_expression= getSdnFileName() + ".SaveAs(" + SEP + _copyName + ")";
	// <_hostDef->_variable>.CloseNoUpdate();
	ReadDNProjectCodePrinter::generateCloseDN( false);
}	