/* NAME:	DN Project Code Printer
 * AUTHOR:	Attila Vizhanyo - viza@isis.vanderbilt.edu
 *
 * Code Printer classes for data networks of udm project.
 */
#ifndef DNPROJECTCODEPRINTER_H
#define DNPROJECTCODEPRINTER_H

#include "DNCodePrinter.h"

class DNProjectCodePrinter : public DNCodePrinter
{
public:
	DNProjectCodePrinter() : _udmSDNSDef( 0), _pushVecExpr( 0), _getDNFromProjectDef( 0)
	{}
	virtual void generate( 
			const std::string& hostFileName,
			const std::string& copyFileName,
			const std::string& meta, 
			const std::string& paradigm, 
			const std::string& root,
			const std::string& dtd
		)
	{
		generateOpen( hostFileName, copyFileName, meta, paradigm, root, dtd, _udmProjectName, _dnsVecName, "Udm::CHANGES_PERSIST_ALWAYS");
		generateClose();
	}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,
		const std::string& meta,
		const std::string& paradigm, 
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) = 0;
	virtual void generateClose() = 0;
	virtual void getOpenCode( PEComposite& addTo) const = 0;
	virtual void getCloseCode( PEComposite& addTo) const = 0;
	virtual void getDnCode( PEComposite& addTo) const
	{
		if ( _getDNFromProjectDef) addTo.add( *_getDNFromProjectDef);
	}
	//
	virtual const std::string& getDN() const
	{
		return _getDNFromProjectDef->_variable;
	}

public:
	static std::string _udmProjectName;
	static std::string _dnsVecName;

protected:
	//
	void generateSDNS( const std::string& hostDN, const std::string& hostFileName, const std::string& meta, const std::string& dnsvecname);
	void generateDNFromProject( const std::string& dnSpecifier, const std::string& meta, const std::string& udmPrj);
	//
	void addSDNSCode( PEComposite& addTo) const
	{
		if ( _udmSDNSDef) addTo.add( *_udmSDNSDef);
		if ( _pushVecExpr) addTo.add( *_pushVecExpr);
	}

private:
	Definition* _udmSDNSDef;
	Expression* _pushVecExpr;
	Definition* _getDNFromProjectDef;
};

class ReadDNProjectCodePrinter : public virtual DNProjectCodePrinter
{
public:
	ReadDNProjectCodePrinter() : _hostDef( 0), _hostOpenExpr( 0), _hostCloseExpr( 0), _hostRTTExpr(0)
	{}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm, 
		const std::string& root,			// unused
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		) ;
	virtual void generateClose();
	virtual void getOpenCode( PEComposite& addTo) const
	{
		addOpenDNCode( addTo);
		addSDNSCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		if ( _hostCloseExpr) addTo.add( *_hostCloseExpr);
	}

protected:
	void generateOpenDN( const std::string& hostFileName, const std::string& meta, const std::string& paradigm, const std::string& dtd, const std::string& mode);
	void generateCloseDN( bool closeWithUpdate); 
	void addOpenDNCode( PEComposite& addTo) const
	{
		if ( _hostDef) addTo.add( *_hostDef);
		if ( _hostOpenExpr) addTo.add( *_hostOpenExpr);
		if ( _hostRTTExpr) addTo.add( *_hostRTTExpr);
	}
	const std::string& getSdnFileName()
	{
		return _hostDef->_variable;
	}
	Definition* _hostDef;
	Expression* _hostRTTExpr;

private:
	Expression* _hostOpenExpr;
	Expression* _hostCloseExpr;
};

class WriteDNProjectCodePrinter : public virtual DNProjectCodePrinter
{
public:
	WriteDNProjectCodePrinter() : _hostDef( 0), _hostCreateExpr( 0), _hostCloseExpr( 0), _hostRTTExpr(0)
	{}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm, 
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		);
	virtual void generateClose();
	virtual void getOpenCode( PEComposite& addTo) const
	{
		addOpenDNCode( addTo);
		addSDNSCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		if ( _hostCloseExpr) addTo.add( *_hostCloseExpr);
	}

protected:
	void generateOpenDN( const std::string& hostFileName, const std::string& meta, const std::string& paradigm, const std::string& root, const std::string& dtd, const std::string& mode);
	void addOpenDNCode( PEComposite& addTo) const
	{
		if ( _hostDef) addTo.add( *_hostDef);
		if ( _hostCreateExpr) addTo.add( *_hostCreateExpr);
		if ( _hostRTTExpr) addTo.add( *_hostRTTExpr);
	}
	const std::string& getSdnFileName()
	{
		return _hostDef->_variable;
	}
	Definition* _hostDef;
	Expression* _hostRTTExpr;
private:
	Expression* _hostCreateExpr;
	Expression* _hostCloseExpr;
};

class ReadWriteDNProjectCodePrinter : public ReadDNProjectCodePrinter
{
public:
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,	// unused
		const std::string& meta,
		const std::string& paradigm, 
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		);
	virtual void generateClose();
	virtual void getOpenCode( PEComposite& addTo) const
	{
		ReadDNProjectCodePrinter::getOpenCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		ReadDNProjectCodePrinter::getCloseCode( addTo);
	}
};

class WriteCopyDNProjectCodePrinter : public ReadDNProjectCodePrinter
{
public:
	WriteCopyDNProjectCodePrinter() : ReadDNProjectCodePrinter(), _hostSaveAsExpr( 0) 
	{}
	virtual void generateOpen(
		const std::string& hostFileName,
		const std::string& copyFileName,
		const std::string& meta,
		const std::string& paradigm, 
		const std::string& root,
		const std::string& dtd,
		const std::string& udmPrj,
		const std::string& dnsvecname,
		const std::string& mode
		);
	virtual void generateClose();
	virtual void getOpenCode( PEComposite& addTo) const
	{
		ReadDNProjectCodePrinter::getOpenCode( addTo);
	}
	virtual void getCloseCode( PEComposite& addTo) const
	{
		if ( _hostSaveAsExpr) addTo.add( *_hostSaveAsExpr);
		ReadDNProjectCodePrinter::getCloseCode( addTo);
	}

protected:

private:
	std::string _copyName;
	Expression* _hostSaveAsExpr;
};


#endif //DNPROJECTCODEPRINTER_H