#include "GRCWrapper.h"
#include "GRCforCG.h"
#include "GReATConfig.h"
#include "GReATUtils.h"
#include "GR.h"
#include <algorithm> // for transform()

void GRCWrapper::createGRC( GReATConfigMeta& metaInfo, const std::string& configPath, GReATConfigMeta::MetaCodeGenModes mcgm) {
	generateGRCCfg( metaInfo, configPath, mcgm);
	GRCC grcc( _cc_cfg);
	grcc.generateLibrary();
}

void GRCWrapper::createSpecCmpCode( GReATConfigMeta& metaInfo, const std::string& configPath, GReATConfigMeta::MetaCodeGenModes mcgm) {
	generateGRCCfg( metaInfo, configPath, mcgm);
	GRCC grcc( _cc_cfg);
	grcc.generateSpecCmpCode( dropPath( dropExtension( _cc_cfg._depFileName)));
}

void GRCWrapper::createVSProj( const std::string& genFileName, GReATConfigMeta& metaInfo, const std::string& configPath, GReATConfigMeta::MetaCodeGenModes mcgm) {
	generateGRCCfg( metaInfo, configPath, mcgm);
	CCodeGenforCGConfig cc_cfg_cg= _cc_cfg;
	cc_cfg_cg._useLib= GReATConfigMeta::isLibraryToGenerate( metaInfo.getMetaCodeGenMode());
	cc_cfg_cg._genFileName= genFileName;
	GRCforCG grcforcg( cc_cfg_cg);
	grcforcg.generateMpc();
	grcforcg.generateVisualStudioProject();
}

void GRCWrapper::createExe( const std::string& genFileName, GReATConfigMeta& metaInfo, const std::string& configPath, GReATConfigMeta::MetaCodeGenModes mcgm) {
	generateGRCCfg( metaInfo, configPath, mcgm);
	CCodeGenforCGConfig cc_cfg_cg= _cc_cfg;
	cc_cfg_cg._useLib= GReATConfigMeta::isLibraryToGenerate( metaInfo.getMetaCodeGenMode());
	cc_cfg_cg._genFileName= genFileName;
	GRCforCG grcforcg( cc_cfg_cg);
	grcforcg.generateMpc();
	grcforcg.generateExe();
}

void GRCWrapper::generateGRCCfg( GReATConfigMeta& metaInfo, const std::string& configPath, GReATConfigMeta::MetaCodeGenModes mcgm) {
	_cc_cfg._transFileName= getFullPath( configPath, metaInfo.getTransformationFileName());
	_cc_cfg._depFileName= _cc_cfg._transFileName;
	_cc_cfg._udmDirName= getPath( getFullPath( configPath, metaInfo.getUdmProjectFileName()), false);
	//
	typedef std::map< std::string, const Udm::DataNetwork*> BackendMap_t;
	BackendMap_t dns;
	metaInfo.getGeneralBackend( dns, configPath);
	StrList_t metanames;
	getFirst( std::make_pair( std::string(), (const Udm::DataNetwork*) 0));	// template instantiation to satisfy the linker
	std::transform( dns.begin(), dns.end(), std::back_inserter( metanames), getFirst< std::string, const Udm::DataNetwork*>);
	if ( metaInfo.isUdmProjectFile()) {
		metanames.push_back( metaInfo.getCrossLinkDiagramName());
	}
	_cc_cfg._metas= metanames;
	_cc_cfg._debug = GReATConfigMeta::isLibraryDebug( mcgm);
	_cc_cfg._dynamic= false;
}

	
