#pragma warning(disable : 4786) 
#include "../GRDebugInterface/GRDebugInterface.h"
#include "GRDBackEnd.h"
#include <iostream>

#include <sstream>
#include <fstream>

namespace GRDI
{

GRDBackEnd::GRDBackEnd() :  m_eventHook(NULL),
                            m_startFunc(nullFunction), 
                            m_suspended(true),
                            m_debugInfo(false),
                            m_tempBreakpoint(false),
                            m_synchStatus(0),
                            m_popCtxNum(0),
                            m_locID(GRDI::invalidLocationID)

{                         
  resetContext();
}
//=========================================

void GRDBackEnd::addNewContext(const FunctionCallDescr& funcCallDescr,
                               const LocationID& currLocationID)
{
  ContextID caller = invalidContextID;
  if (m_context.size())
    caller = m_context.size() - 1;
    
  m_context.push_back(ContextDescr(m_context.size(), funcCallDescr, caller, currLocationID));
}

//=========================================

void GRDBackEnd::resetContext()
{
  if (m_context.size())
  {
    m_popCtxNum = 0;
    m_popEventCtxID = invalidContextID;
    m_context.clear();
  }
}
//=========================================
void GRDBackEnd::clearSource()
{
  if (m_id2loc.size())
  {
    resetContext();
    m_rootFuncs.clear();
    m_functions.clear();
    m_id2loc.clear();
    m_loc2id.clear();
    m_breakpoints.clear();
  }
}
//=========================================

void GRDBackEnd::refreshCurrContext(const FunctionCallDescr& funcCallDescr, 
                                    const LocationID& currLocationID)
{
  ContextDescr&  ctx = m_context.back();

  if (ctx.m_caller != funcCallDescr.m_caller)
    throw std::invalid_argument("improper context management");

  ctx.m_varValMap = funcCallDescr.m_varValMap;
//x  VariableValueMap& valMap = ctx.m_varValMap;
//x  VariableValueMap::const_iterator it = funcCallDescr.m_varValMap.begin(); 
//x  for (;it != funcCallDescr.m_varValMap.end(); ++it)
//x     valMap[it->first] = it->second;

//  ctx.m_varValMap.insert(funcCallDescr.m_varValMap.begin(), funcCallDescr.m_varValMap.end());
  ctx.m_currLocationID = currLocationID;


}
//=========================================
void GRDBackEnd::popContext()
{

  if (m_popCtxNum > 0)
  {
    m_context.resize(m_context.size() - m_popCtxNum);
    m_popCtxNum = 0;
    if (m_popEventCtxID > 0)
    {
      Event ev;
      ev.m_kind = EVENT_CONTEXT_POP;
      getCurrentContextID(ev.m_u.m_ctx.m_ctxID);
      getCurrentLocation(ev.m_u.m_ctx.m_ctxID, ev.m_u.m_ctx.m_loc);
      m_eventSet.push_back(ev);
      m_popEventCtxID = invalidContextID; //invalidate
    }
  }
}

//=========================================
void GRDBackEnd::resetEventSet()
{
  m_eventSet.clear();
}

//=========================================
void GRDBackEnd::checkBreakpoints(const LocationID& id)
{

  LocationIDSet::const_iterator itb = m_breakpoints.find(id);
  if ((itb != m_breakpoints.end()) || m_tempBreakpoint)
  {
    Event ev;
    ev.m_kind = EVENT_BREAKPOINT;

    ev.m_u.m_breakpoint.m_loc = id;
    ev.m_u.m_breakpoint.m_pause = m_tempBreakpoint;
    m_eventSet.push_back(ev);
  }
  
}
//=========================================

LocationID GRDBackEnd::findBeginLocation(const IFunctionPtr& func) 
{
  return findLocation(func->getID(), FunctionBegin);
}
//=========================================

LocationID GRDBackEnd::findEndLocation(const IFunctionPtr& func) 
{
  return findLocation(func->getID(), FunctionEnd);
}
//=========================================

LocationID GRDBackEnd::findCallLocation(const IFunctionPtr& func) 
{
  return findLocation(func->getID(), FunctionCall);
}
//=========================================
LocationID GRDBackEnd::findLocation(const FunctionID& id,
                                    const LocationType& type) 
{
  
  LocLocIDMap::const_iterator it = 
    m_loc2id.find(FunctionIDLocationTypePair(id, type));
 
  if (it == m_loc2id.end())
  {
    std::stringstream str;
    str << "invalid location" << std::endl;
    throw std::invalid_argument(str.str());
  }

  return it->second;
}

//=========================================
void GRDBackEnd::orderCalls()
{
  FunctionDescrMap::iterator it = m_functions.begin();
  for(; it != m_functions.end(); ++it)
  {
    orderFunctionCalls(it->second);
  }

}
 //=========================================
void GRDBackEnd::setLocations(const LocationID& parLocID,
                              const IFunctionPtr& func)
{
 
  FunctionDescrMap::iterator itD = m_functions.find(func);
  if (itD == m_functions.end())
       throw std::invalid_argument("invalid function");

  FunctionDescr& funcDescr = itD->second;
  if (funcDescr.m_ref == IFunctionPtr())
  {
    LocationIDVec& callIDs = funcDescr.m_callLocs;
    callIDs.push_back(addLocation(parLocID, Location(func, func, FunctionBegin)));

    const FunctionTable& calls = funcDescr.m_calls;
  
    FunctionTable::const_iterator it = calls.begin();
    for (;it != calls.end(); ++it)
    {
     const IFunctionPtr& childFunc = *it;
     callIDs.push_back(addLocation(parLocID, 
       Location (funcDescr.m_func, 
                 childFunc, 
                 FunctionCall)));
    }

    callIDs.push_back(addLocation(parLocID, Location(func, func, FunctionEnd)));

    it = calls.begin();
    for (;it != calls.end(); ++it)
    {
      const IFunctionPtr& childFunc = *it;
      const LocationID& childLocID = *(callIDs.begin() + (it -calls.begin()));
      setLocations(childLocID, childFunc);
    }
  }
  else
    m_refMap[findCallLocation(func)] =   funcDescr.m_ref;

}
//=========================================
void GRDBackEnd::resetFlags()
{
  m_tempBreakpoint = false;
}
};//namespace GRDI

