#pragma warning(disable : 4786) 
#include "GRDFrontEnd.h"
#include "../GRDebugInterface/GRDebugInterface.h"
#include <stdexcept>
#include <sstream>

//#include "../Log/Log.h"


namespace GRDI
{
//======================================
GRDFrontEnd::GRDFrontEnd() : 
    m_backEnd(0),
    m_events(100),
    m_notify(0),
    m_up(false),
    m_runCmd(false),
    m_nextCmd(false),
    m_stepCmd(false),
    m_stepOutCmd(false),
    m_stopCmd(false)
{
}
//======================================
void GRDFrontEnd::handleBreakpoint(const BreakpointEventData& breakpoint,
                                   bool   pause)
{
  m_backEnd->suspend();

//x  if (m_nextCmd)
//x  {
//x    m_notify->stepIn(breakpoint.m_loc);
//x    m_backEnd->clearBreakpoint(breakpoint.m_loc);
//x    m_nextCmd = false;
//x  }
//x  else
    m_notify->breakpoint(breakpoint.m_loc, pause);
 
 
}
//======================================
void GRDFrontEnd::resetFlags()
{
    m_runCmd = false;
    m_nextCmd = false;
    m_stepCmd = false;
    m_stepOutCmd = false;
    m_stopCmd = false;
}
//========== GRDebugAPI
//======================================
void GRDFrontEnd::setBackEnd(DebugInterface* backEnd)
{

  if (backEnd == NULL)
    throw std::invalid_argument("DebugInterface null pointer");

  m_backEnd = backEnd;
  m_backEnd->setEventHook((EventHook*)this) ;
  
}
//======================================
void GRDFrontEnd::setNotificationInterface(GRDebugNotification* notify)
{
  m_notify = notify;
}
//======================================
void GRDFrontEnd::init()
{
  m_backEnd->suspend();
  
}
//======================================
void GRDFrontEnd::run()
{
#ifndef _DEBUG 
try
{
#endif
  

  bool finish = false;
  m_up = true;
  while (!finish)
  {
    EventSet es = m_events.remove();

    bool go = true;
    
    m_notify->eventHandligStart();

    bool dropEvents = false;
    EventSet::const_iterator it = es.begin();
    for (;it != es.end(); ++it)
    {
      const Event& event = *it;
    
      if (event.m_kind == EVENT_GR_ENGINE_DOWN)
      {
        m_notify->engineIsDown();
        go = true;
        dropEvents = true;
        finish = true;
      }
      else
      if (event.m_kind == EVENT_EXCEPTION)
      {
        m_notify->exception(event.m_u.m_exception.m_loc);
        go = false;
        dropEvents = true;
      }
      else
      if (event.m_kind == EVENT_GR_LOAD)
      {
        m_notify->loaded();
        dropEvents = true;
        go = false;
      }
      else
      if (event.m_kind == EVENT_GR_CLOSE)
      {                  
        m_notify->closed();
        dropEvents = true;
        go = false;
      }
      if (dropEvents)
        break;
   }


    if (dropEvents)
    {
      resetFlags();
    }
    else
    {
      go = true;
      EventSet::const_iterator it = es.begin();
      for (;it != es.end(); ++it)
      {
        const Event& event = *it;
    
        if (event.m_kind == EVENT_BREAKPOINT)
        {
          handleBreakpoint(event.m_u.m_breakpoint, event.m_u.m_breakpoint.m_pause);
          go = false;
        }
        else
        if (event.m_kind == EVENT_FUNCTION_ENTRY)
        {
          m_notify->stepIn(event.m_u.m_singleStep.m_loc);
        }
        else
        if (event.m_kind == EVENT_FUNCTION_EXIT)
        {
          //m_notify->stepOut(event.m_u.m_singleStep.m_loc);
        }
        else
        if (event.m_kind == EVENT_FUNCTION_STEP)
        {
          if (m_stepCmd)
          {
            m_stepCmd = false;
            m_notify->step(event.m_u.m_singleStep.m_loc);
            go = false;
          }
          else
          if (m_nextCmd)
          {
            ContextID curr;
            m_backEnd->getCurrentContextID(curr);
            if (curr <= m_nextCtxID)
            {
               m_notify->step(event.m_u.m_singleStep.m_loc);
               go = false;
               m_nextCmd = false;
            }
          }
          else
          if (m_stepOutCmd)
          {
            ContextID curr;
            m_backEnd->getCurrentContextID(curr);

            if (curr < m_stepOutCtxID)
            {
               m_notify->step(event.m_u.m_singleStep.m_loc);
               go = false;
               m_stepOutCmd = false;
            }
          }
        }
        else
        if (event.m_kind == EVENT_CONTEXT_POP)
        {
  //x        m_notify->ctxPop(event.m_u.m_ctx.m_loc);
  //x        go &= false;
        }
        else
        if (event.m_kind == EVENT_GR_START)
        {
          m_notify->start();
          go = true;
        }
        else
        if (event.m_kind == EVENT_GR_FINISH)
        {
          m_notify->finish();
          go = false;
        }
        else
        if (event.m_kind == EVENT_GR_STOPPED)
        {
          if (!m_runCmd)
            m_notify->stopped();
          else
            m_runCmd = false;
          go = false;
        }
        else
        if (event.m_kind == EVENT_GR_ENGINE_UP)
        {
          m_notify->engineIsUp();
          go = false;
        }
        else
        if (event.m_kind == EVENT_GR_SAVE)
        {
          m_notify->saved();
          go = false;
        }
        else
        if (event.m_kind == EVENT_GR_PROGRESS)
        {
          m_notify->progress(event.m_u.m_progress.m_prgs);
          go &= true;
        }
      }
    }

    if (go)
      m_backEnd->resume();

    m_notify->eventHandligFinish();
  }  
  m_up = false;
#ifndef _DEBUG 
}
catch(...)
{
  std::cout << "GRDFrontEnd exception !!!" << std::endl;
}
#endif
}

//======================================
void GRDFrontEnd::wait_up()
{
  while(!m_up);
}
//======================================
void GRDFrontEnd::stop()
{
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::clearAllBreakpoints()
{
  m_backEnd->clearAllBreakpoints();
}
//======================================
bool GRDFrontEnd::setBreakpointCmd(const LocationID& loc)
{
 if (m_backEnd->setBreakpoint(loc))
    return false;

  return true;
}
//======================================
bool GRDFrontEnd::clearBreakpoint(const LocationID& loc)
{
  if (m_backEnd->clearBreakpoint(loc))
    return false;

  return true;
}
//======================================
void GRDFrontEnd::saveCmd()
{
  m_backEnd->save();
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::loadCmd(const std::string& pn)
{
  m_backEnd->load(pn);
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::closeCmd()
{
  m_backEnd->close();
  m_backEnd->resume();
}

//======================================
void GRDFrontEnd::startCmd()
{
  m_backEnd->execute();
  m_backEnd->resume();
}

//======================================
void GRDFrontEnd::runCmd()
{
  m_backEnd->run();
  m_backEnd->resume();
}

//======================================
void GRDFrontEnd::stopCmd()
{
  m_backEnd->stop();
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::pauseCmd()
{
  m_backEnd->pause();
  m_backEnd->resume();
}

//======================================
void GRDFrontEnd::nextCmd()
{
  m_nextCmd = true;
  m_backEnd->getCurrentContextID(m_nextCtxID);
  m_backEnd->run();
  m_backEnd->resume();
  
}
//======================================
void GRDFrontEnd::stepCmd()
{
  m_stepCmd = true;
  m_backEnd->run();
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::stepOutCmd()
{
  m_stepOutCmd = true;
  m_backEnd->getCurrentContextID(m_stepOutCtxID);
  m_backEnd->run();
  m_backEnd->resume();
}
//======================================
void GRDFrontEnd::quitCmd()
{
  m_backEnd->quit();
  m_backEnd->resume();
}
//======================================
DebugInterface& GRDFrontEnd::getDebugInterface()
{
  return (DebugInterface&) (*m_backEnd);
}
//========== EventHook
//======================================
Error GRDFrontEnd::operator()(EventSet event)
{
  m_backEnd->suspend();
  m_events.insert(event);
  return GRDI_ERROR_NONE;

}


};