#pragma warning(disable : 4786) 

#include <fstream>
#include "../GRLauncher/GRDebuggeeAPI.h"
#include "GRDebugger.h"
#include "GRDebuggerHelp.h"


// #include "../Log/Log.h"
// extern Log lout;

namespace GRDI
{

//=========================================
GRDebugger::GRDebugger() : m_frontEnd(0),
                           m_view(0),
                           m_eventHandlig(false), 
                           m_isUp(false),
                           m_loaded(false),
                           m_exception(false),
                           m_logging(false),
                           m_progress(0)
{
}
//=========================================
void GRDebugger::setFrontEnd(GRDI::IGRDFrontEnd* frontEnd)
{
  m_frontEnd = frontEnd;
  m_debugInterface = &(m_frontEnd->getDebugInterface());
}
//=========================================
void GRDebugger::setIGRDebuggerView(IGRDebuggerView* view )
{
  m_view = view;
}
//=========================================
GRDI::GRDebugNotification* GRDebugger::getNotificationInterface()
{
  return static_cast<GRDI::GRDebugNotification*>(this);
}

//=========================================
IGRDebuggerCtrl* GRDebugger::getIGRDebuggerCtrl()
{
  return static_cast<IGRDebuggerCtrl*> (this);
}
//=========================================
void GRDebugger::init(const std::string& fn)
{
  m_dataFName = fn;
  readHelp(std::ifstream("GRDebuggerHelp.txt"));
}

//=========================================
void GRDebugger::readHelp(std::istream& out)
{
  m_helpTxt = GRDI::help;
}
//=========================================
void GRDebugger::buildFunctionSource(const LocationID& locID,
                                        std::stringstream& str)
{
  Location loc;
  if (m_debugInterface->getLocationFromID(locID, loc))
    throw std::invalid_argument("invalid location");

  if (loc.type() == FunctionCall)  
  {
    FunctionDescr descr;
    if (m_debugInterface->getFunctionDescr(loc.called(), descr))
       throw std::invalid_argument("invalid function");

    m_view->buildFunctionSourceImpl(loc, descr, str);

    LocationIDVec::const_iterator itC = descr.m_callLocs.begin();
    for(; itC != descr.m_callLocs.end(); ++itC)
      buildFunctionSource(*itC, str);
  }
}


//=========================================
void GRDebugger::buildSource()
{
  
  std::stringstream str;
  m_view->buildSourceImplBegin(str);


  LocationIDVec rootLocs;
  m_debugInterface->getRootLocations(rootLocs);

  LocationIDVec::const_iterator it = rootLocs.begin();
  for(; it != rootLocs.end(); ++it)
  {
    const LocationID& locID = *it;
    buildFunctionSource(locID, str);
  }
  
  m_view->buildSourceImplEnd(str);
}

//=========================================
void GRDebugger::markFuncLocation(std::stringstream& str, 
                                  const GRDI::LocationIDSet& locIDs,
                                  int mark)
{

  GRDI::LocationIDSet::const_iterator it =  locIDs.begin();

  IGRDebuggerView::FuncLocationMarkDescrVec locs;
  for(; it != locIDs.end(); ++it)
  {
    IGRDebuggerView::FuncLocationMarkDescr locDescr;
    if (m_debugInterface->getLocationFromID(*it, locDescr.m_loc))
      throw std::invalid_argument("invalid location");

    if (locDescr.m_loc.caller())
    {
      FunctionDescr descr;
      if (m_debugInterface->getFunctionDescr(locDescr.m_loc.caller(), locDescr.m_descr))
         throw std::invalid_argument("invalid function");
    }

    locDescr.m_mark = mark;
    locs.push_back(locDescr);
  }

  m_view->markFuncLocationImpl(str, locs);
}
//=========================================
void GRDebugger::logLocation(std::stringstream& str, 
                             const GRDI::LocationID& locID)
{
    GRDI::Location location;
    
    if(m_debugInterface->getLocationFromID(locID,location))
      throw std::invalid_argument("invalid location");
    ContextID ctxID; 
    if(m_debugInterface->getCurrentContextID(ctxID))
      throw std::invalid_argument("invalid location");
    std::string sp;
    //sp.resize(std::string::size_type(2 * ctxID));
    str << sp << *(location.caller()) << std::endl;
}
//=========================================
void GRDebugger::printContext(const ContextID& ctxID, 
                             std::ostream& str,
                             bool exception)
{
  ContextDescr contextDescr;
  if (m_debugInterface->getContextDescr(ctxID, contextDescr))
    throw std::invalid_argument("invalid context");

  m_view->printContextImpl(contextDescr, str, exception);
}

//=========================================
void  GRDebugger::printContextStack(bool exception)
{
  std::stringstream str;
  m_view->printContextStackImplBegin(str, exception);
  
  int size = 0;
  m_debugInterface->getContextStacktSize(size );

  if (size)
  {
    ContextID ctxID; 
    m_debugInterface->getCurrentContextID(ctxID);

    do
    {
      printContext(ctxID, str, exception);
      m_debugInterface->getCallerContext(ctxID,  ctxID);
    }
    while(ctxID != invalidContextID);
    
  }
  else
    str <<  "the stack is empty\n";

  m_view->printContextStackImplEnd(str, exception);
}
//=========================================
void GRDebugger::showAllBreakPoints()
{
  LocationIDSet breakpoints;
  m_debugInterface->getBreakPoints(breakpoints);

  std::stringstream str;
  if (breakpoints.size())
  {
    str << "Breakpoints :" << std::endl;
    markFuncLocation(str, breakpoints, IGRDebuggerView::BREAK);
  }
  else
    str << "No breakpoint" << std::endl;
  m_view->print(str.str());
}
//=========================================
void GRDebugger::whereCmd()
{
  printContextStack(m_exception);
}
//=========================================
void GRDebugger::localsCmd()
{
  std::stringstream str;

  ContextID ctxID; 
  if (!m_debugInterface->getCurrentContextID(ctxID))
  {
    printContext(ctxID, str);

    m_view->print(str.str());
  }
  else
    m_view->print("there is no context");

}

//=========================================
void GRDebugger::listCmd()
{
  m_view->printSource();
}
//=========================================
void GRDebugger::saveCmd()
{
  m_exception = false;
  m_saved = false;
  m_view->print("Saving the backend ...\n");

  m_frontEnd->saveCmd();
}

//==============Printing the load command===========================
void GRDebugger::loadCmd(const std::string& cmd)
{
  m_exception = false;
  if ((cmd == "load") && !m_dataFName.size())
  {
    m_view->print("filename should be given for load command");
    return;
  }
  else
  if ((cmd != "load"))
  {
    int i0 = cmd.find( "load" );
    int i = cmd.find_first_not_of(" ", i0 + 4 );
    m_dataFName = cmd.substr(i); 
  }
  

  m_loaded = false;
  m_view->print("Loading the program using "+ m_dataFName + "...\n");

  m_frontEnd->loadCmd(m_dataFName);
//   while(!m_loaded && !m_exception); 
// 
//   if (!m_exception)
//   {
//     buildSource();
//     listCmd();
//   }
}
//=========================================
void GRDebugger::closeCmd()
{
  if (m_loaded)
  {
    m_loaded = false;
  
    m_view->print("Closing the program "+ m_dataFName + "...\n");

    m_frontEnd->closeCmd();
  }
}
//=========================================
void GRDebugger::clearCmd(const std::string& cmd)
{
  if (cmd == "clear")
    showAllBreakPoints();
  else
  if (cmd == "clear all")
  {

    m_view->print("all breakpoints have been removed\n");

    LocationIDSet breakpoints;
    std::stringstream strm;
    strm << "all the breakpoint has been cleared " << std::endl;

    if (m_debugInterface->getBreakPoints(breakpoints))
      throw std::invalid_argument("error in getting breakpoints");

    markFuncLocation(strm, breakpoints, IGRDebuggerView::BREAK | IGRDebuggerView::SET_BREAK );
    m_frontEnd->clearAllBreakpoints();
  }
  else
  {
    int i = cmd.find("clear");
    int bp = atoi(cmd.substr(i+5).c_str()); 

    if (!m_frontEnd->clearBreakpoint(bp))
      m_view->print("unknown source location\n");
    std::stringstream strm;
    strm << "breakpoint has been removed at location " << bp << std::endl;
    m_view->print(strm.str());
    LocationIDSet breakpoints;
    breakpoints.insert(bp);
    markFuncLocation(strm, breakpoints, IGRDebuggerView::BREAK | IGRDebuggerView::SET_BREAK );
  }
}
//=========================================
void GRDebugger::setCmd(const std::string& cmd)
{
     
  int i = cmd.find("set");
  int bp = atoi(cmd.substr(i+3).c_str()); 

  if (!m_frontEnd->setBreakpointCmd(bp))
    m_view->print("unknown source location\n");
  else
  {
    std::stringstream strm;
    strm << "a breakpoint has been set at location " << bp << std::endl;
    m_view->print(strm.str());
    LocationIDSet breakpoints;
    breakpoints.insert(bp);
    markFuncLocation(strm, breakpoints, IGRDebuggerView::BREAK | IGRDebuggerView::SET_BREAK );
  }
}
//=========================================
void GRDebugger::logCmd(const std::string& cmd)
{
  if ((cmd == "log") || 
      (cmd == "log on"))
  {
    m_logging = true;
    m_view->print("Function call logging is on.\n");
  }
  else
  if (cmd == "log off")
  {
    m_logging = false;
    m_view->print("Function call logging is off.\n");
  }
}
//=========================================
void GRDebugger::progressCmd()
{
  m_view->printCurrentProgress(m_progress);
}
//=========================================
void GRDebugger::evalCommand(const std::string& cmd)
{
  if (cmd == "help" || cmd == "h") 
  {
    m_view->print(m_helpTxt);
  }
  else  if (cmd.find("load") != std::string::npos )
  {
    loadCmd(cmd);
  }
  else  if ((cmd == "quit") || (cmd == "q")) 
  {
    m_frontEnd->quitCmd();
    m_view->print("Debugger has been quitted");
    m_quit = true;
  }
  else  if (!m_loaded)
  {
    m_view->print("there is no program to debug");
  }
  else  if ((cmd == "list") || (cmd == "l"))
  {
    listCmd();
  }
  else  if (cmd == "start")
  {
    m_exception = false;
    m_frontEnd->startCmd();
  }
  else  if (cmd == "stop")
  {
    m_frontEnd->stopCmd();
  }
  else  if (cmd == "pause")
  {
    m_frontEnd->pauseCmd();
  }
  else  if ((cmd == "run") || (cmd == "r"))
  {
    m_frontEnd->runCmd();
    m_exception = false;
    m_view->print("running ...\n");
  }
  else  if ((cmd == "step") ||(cmd == "s"))
  {
    m_exception = false;
    m_frontEnd->stepCmd();
  }
  else  if ((cmd == "step out") ||  (cmd == "so"))
  {
    m_exception = false;
    m_frontEnd->stepOutCmd();
  }
  else  if ((cmd == "next") ||   (cmd == "n"))
  {
    m_exception = false;
    m_frontEnd->nextCmd();
  }
  else  if ((cmd == "where") ||  (cmd == "w"))
  {
    whereCmd();
  }
  else  if ((cmd == "print") ||  (cmd == "p"))
  {
    localsCmd();
  }
  else  if (cmd.find("set") != std::string::npos )
  {
    setCmd(cmd);
  }
  else  if (cmd.find("clear") != std::string::npos )
  {
    clearCmd(cmd);
  }
  else  if (cmd.find("log") != std::string::npos )
  {
    logCmd(cmd);
  }
  else  if (cmd.find("save") != std::string::npos )
  {
    saveCmd();
  }
  else  if (cmd.find("progress") != std::string::npos )
  {
    progressCmd();
  }
  else  if (cmd.find("close") != std::string::npos )
  {
    closeCmd();
  }
  else
    m_view->print("unknown command\n");

}

//=========================================
//========== IGRDebuggerCtrl

void GRDebugger::startCtrl()
{
  Lock lk(m_mutexCtrl);  
  while (!m_isUp);

  m_view->print(intro);

  if(m_dataFName.size())
    loadCmd("load");
  
  m_quit = false;
}
 
//=========================================
bool GRDebugger::stepCtrl(const std::string& cmd)
{
  Lock lk(m_mutexCtrl);  
  
  evalCommand(cmd);
  
  while (m_eventHandlig);

  return m_quit;
}
//=========================================
void GRDebugger::stopCtrl()
{
  Lock lk(m_mutexCtrl);  
  while(m_isUp);

  m_view->print("Bye.");
  //lout.print(1,"GRDebugger","debugger has been stopped\n");
}


//========== GRDebugNotification
//======================================

void GRDebugger::eventHandligStart()
{
   Lock lk(m_mutexNotification);  
   m_eventHandlig = true;
}
//======================================
void GRDebugger::eventHandligFinish()
{
   Lock lk(m_mutexNotification);  
   m_eventHandlig = false;
}
//======================================
void GRDebugger::breakpoint(const GRDI::LocationID& locID, bool pause)
{
  Lock lk(m_mutexNotification);  

  LocationIDSet locs;
  locs.insert(locID);

  std::stringstream str;
  if (!pause)
  {
    str << "a breakpoint has been hit at location " << locID;
    markFuncLocation(str, locs, IGRDebuggerView::HIT_BREAK | IGRDebuggerView::CURR);
  }
  else
  {
    str << "the program has been paused at location " << locID;
    markFuncLocation(str, locs, IGRDebuggerView::CURR);
  }
  
  m_view->hitBreakpoint();

  str << std::endl;
  m_view->print(str.str());
}
//======================================
void GRDebugger::stepIn(const GRDI::LocationID& loc)
{
  Lock lk(m_mutexNotification);  
  if (m_logging)
  {
    std::stringstream str;
    logLocation(str, loc);
    m_view->print(str.str());
  }
}
//======================================
void GRDebugger::step(const GRDI::LocationID& loc)
{
  Lock lk(m_mutexNotification);  

  std::stringstream str;
//  str << "step on " << loc << std::endl;
  LocationIDSet locs;
  locs.insert(loc);

  markFuncLocation(str,locs, IGRDebuggerView::CURR);
  m_view->print(str.str());
  m_view->step();
}
//======================================
void GRDebugger::stepOut(const GRDI::LocationID& loc)
{
  Lock lk(m_mutexNotification);  
  std::stringstream str;
  str << "step out " << loc << std::endl;
  m_view->print(str.str());
}
//======================================
void GRDebugger::ctxPop(const GRDI::LocationID& loc)
{
  Lock lk(m_mutexNotification);  
  std::stringstream str;
  str << "context pop step on " << loc << std::endl;
  m_view->print(str.str());
}
//======================================
void GRDebugger::stopped()
{
  Lock lk(m_mutexNotification);  
  m_view->print("program has been stopped\n");
  m_view->stopped();

}
//======================================
void GRDebugger::exception(const GRDI::LocationID& locID)
{
  Lock lk(m_mutexNotification);  
  std::stringstream stream;

  stream << std::endl << "Exception is thrown in GReAT " << std::endl;

  if (locID != invalidLocationID)
  {
    stream << "at location " << locID << std::endl;
  }
  std::string str;
  m_debugInterface->getLastException(str);

  stream << str;
  
  m_view->print(stream.str());
  m_exception = true;

//   if (m_loaded)
//     markFuncLocation(stream, locID, IGRDebuggerView::CURR);
  m_view->exception();
}

//======================================
void GRDebugger::start()
{
  Lock lk(m_mutexNotification);  

  m_view->print("program has been started\n");
}
//======================================
void GRDebugger::finish()
{
  Lock lk(m_mutexNotification);  
  m_view->print("program has been finished\n");
  m_view->finished();
}

//======================================
void GRDebugger::engineIsUp()
{
  Lock lk(m_mutexNotification);  
//  m_view->print("gr engine is up\n");
  m_isUp = true;
  
}

//======================================
void GRDebugger::engineIsDown()
{
  Lock lk(m_mutexNotification);  
//  m_view->print("gr engine is down\n");
  m_isUp = false;
}

//======================================
void GRDebugger::saved()
{
  Lock lk(m_mutexNotification);  
  m_view->print("backend has been saved\n");
  m_view->saved();
}

//======================================
void GRDebugger::loaded()
{
  Lock lk(m_mutexNotification);  
  if (!m_exception)
  {
    buildSource();
    listCmd();
  }

  m_view->print("program has been loaded\n");
  m_view->loaded();
  m_loaded = true;
}
//======================================
void GRDebugger::closed()
{
  Lock lk(m_mutexNotification);  
  m_view->print("program has been closed\n");
  m_view->closed();
}

//======================================
void GRDebugger::progress(double prg)
{
  Lock lk(m_mutexNotification);  
  m_progress = prg;
  m_view->progressReport(prg);
}
};//namespace GRDI

