// GRDebuggerGUIDoc.h : interface of the CGRDebuggerGUIDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRDEBUGGERGUIDOC_H__4FBFE10A_CD8C_4E62_9C32_D78057C8724F__INCLUDED_)
#define AFX_GRDEBUGGERGUIDOC_H__4FBFE10A_CD8C_4E62_9C32_D78057C8724F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../GRLauncher/GRDebuggerAPI.h"
#include "../GRDebugger/IGRDebuggerCtrl.h"
#include "../GRDebugger/IGRDebuggerView.h"
#include "../GRLauncher/GRLauncher.h"

#include "SourceTxtView.h"
#include "StackView.h"


#include "MainFrm.h"

#include <boost/thread/mutex.hpp>
typedef boost::mutex::scoped_lock Lock;



class CGRDebuggerGUIDoc : public CDocument,
                           public GRDI::IGRDebuggerView,
                           public GRDebuggerConsolAPI
{
protected: // create from serialization only
	CGRDebuggerGUIDoc();
	DECLARE_DYNCREATE(CGRDebuggerGUIDoc)
  void stackView();
  void enableCtrl();  
  void disableCtrl();  

  GRDI::IGRDebuggerCtrl* m_debuggerCtrl;
public : 


  CString           m_lpszPathName;
  CMainFrame*       m_pMainFrame; 
  boost::mutex      m_mutexPrint;

  bool             m_ctrlEnabled;
  bool             m_pauseEnabled;
  bool             m_stopEnabled;
  bool             m_stackViewChanged;
  bool             m_loading;
  bool             m_closing;
  bool             m_running; 
  GRDI::ContextID m_prevCtxId;


  
  GRDI::IGRDebuggerView::FuncLocationMarkDescrVec m_markLocs;
  bool m_sourceViewChanged; 

public:
  //======IGRDebuggerView
  void print(const std::string&);

  void buildFunctionSourceImpl( const GRDI::Location& loc,
                              const GRDI::FunctionDescr& descr, 
                              std::stringstream& str);

  void markFuncLocationImpl(  std::stringstream& str,
                              GRDI::IGRDebuggerView::FuncLocationMarkDescrVec& locs);

  void buildSourceImplBegin(std::stringstream& str);
  void buildSourceImplEnd(std::stringstream& str);
  void printSource();

  void printContextImpl(const GRDI::ContextDescr& contextDescr, 
                        std::ostream& str,
                        bool exception);

  void printContextStackImplBegin(std::ostream& str, bool exception);
  void printContextStackImplEnd(std::ostream& str, bool exception);

  void stopped();
  void exception();
  void finished();
  void loaded();
  void saved();
  void closed();  


  void hitBreakpoint();
  void step();
  void progressReport(double);
  void printCurrentProgress(double);

  void setIGRDebuggerCtrl(GRDI::IGRDebuggerCtrl* debuggerCtrl);
  GRDI::IGRDebuggerView* getIGRDebuggerView();



  GRLauncher   m_launcher;
  void initDebugger();
  
  void closeDebugger();

public:
    void handleException();
    void update();
    void viewUpdate();
    void startDebugger();
    void viewsChanged(){ m_stackViewChanged =  true;}
    void debuggerCtrl(const std::string&);
    void executeCmd();

public:
    CView * m_logView;
    SourceTxtView*    m_srcView;
    StackView*        m_stackView;


    CString  m_txt; 
    std::string m_debuggerCmd; 

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGRDebuggerGUIDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void OnCloseDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CGRDebuggerGUIDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CGRDebuggerGUIDoc)
	afx_msg void OnStep();
	afx_msg void OnUpdateStep(CCmdUI* pCmdUI);
	afx_msg void OnStepover();
	afx_msg void OnUpdateStepover(CCmdUI* pCmdUI);
	afx_msg void OnStart();
	afx_msg void OnUpdateStart(CCmdUI* pCmdUI);
	afx_msg void OnStop();
	afx_msg void OnUpdateStop(CCmdUI* pCmdUI);
	afx_msg void OnStepout();
	afx_msg void OnUpdateStepout(CCmdUI* pCmdUI);
	afx_msg void OnPause();
	afx_msg void OnUpdatePause(CCmdUI* pCmdUI);
	afx_msg void OnCont();
	afx_msg void OnUpdateCont(CCmdUI* pCmdUI);
  afx_msg void OnVariablePopUp( UINT nID );
  afx_msg void OnGotoLocationPopUp( UINT nID );
	afx_msg void OnGrClose();
	afx_msg void OnUpdateGrClose(CCmdUI* pCmdUI);
	afx_msg void OnGrOpen();
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileOpen(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_GRDEBUGGERGUIDOC_H__4FBFE10A_CD8C_4E62_9C32_D78057C8724F__INCLUDED_)
