#pragma warning(disable : 4786) 
#include <boost/thread/thread.hpp>
#include "GRLauncher.h"
#include "../GRDBackEnd/GRDBackEndFactory.h"
#include "../GRDFrontEnd/GRDFrontEndFactory.h"

#include "../GRDBackEnd/IGRDBackEnd.h"
#include "../GRDFrontEnd/IGRDFrontEnd.h"

#include "GRDebuggeeAPI.h"
#include "GRDebuggerAPI.h"
#include "../GRDebugger/GRDebuggerFactory.h"

#include "GRDebuggerAPI.h"
#include "../GReATDebugger/GReATDebuggerFactory.h"

#include <direct.h>

std::string GRLauncher::m_usageTxt = "usage: grd.exe  paramFileName [-nd | -d]" ;
//=====================================
bool GRLauncher::setPars(int argc, const char* argv[])
{
  m_invalidPar = false;
  m_debugMode = false;
  if (argc > 1)
  {
    char buffer[200];
    _getcwd(buffer, 200);
    m_wd = std::string(buffer);

    if (argc == 2) 
    {
        m_dataFName = std::string(argv[1]);
    }
    else
    if ((argc == 3))
    {
      m_dataFName = std::string(argv[1]);
      std::string aa(argv[2]);
      if (aa == "-nd")
        m_debugMode = false;
      else
      if ((aa == "-d") || (aa == "-c"))
        m_debugMode = true;
      else
        m_invalidPar  = true;
    }
    else
      m_invalidPar  = true;
  }
  else
    m_invalidPar  = true;
  
  if (!m_debugMode && m_dataFName.size() <= 0)
    m_invalidPar = true;

  if (m_invalidPar)
    std::cout << m_usageTxt << std::endl;
  return !m_invalidPar;
}
//=====================================
bool GRLauncher::init(GRDebuggerConsolAPI* debuggerConsol)
{
  if (!m_invalidPar)
  {
    try
    {
//       debuggerConsol->setIGRDebuggerCtrl(0);
    m_debuggee = GReATDebuggerFactory::create_new_thread();
    m_debuggee->wait_up();
    if (m_debugMode)
    {
      m_front =  GRDI::GRDFrontEndFactory::create_new_thread();
      m_front->wait_up();
      m_debugger = GRDI::GRDebuggerFactory::create();


      m_debugger->setIGRDebuggerView(debuggerConsol->getIGRDebuggerView());
      debuggerConsol->setIGRDebuggerCtrl(m_debugger->getIGRDebuggerCtrl());

      m_back =  GRDI::GRDBackEndFactory::create();

      m_front->setBackEnd(m_back->getDebugInterface());

      m_debuggee -> setGRDBackEnd(m_back);
      m_debuggee -> setWD(m_wd);

      m_front->setNotificationInterface(m_debugger->getNotificationInterface());
      m_debugger->setFrontEnd(m_front);
      m_debugger->init(m_dataFName);
    }
    }
    catch(...)
    {
      std::cout << "grd launch exception" << std::endl;
      return false;
    }
  }
  return true;
}
//=====================================
GRLauncher::GRLauncher() :
        m_front(0),
        m_debugger(0),
        m_back(0),
        m_debuggee(0),
        m_debugMode(true),
        m_invalidPar(false)
{
}
//=====================================
GRLauncher::~GRLauncher()
{
  if (m_front)
    delete m_front, m_front  = 0;

  if (m_back)
    delete m_back, m_back  = 0;

  if (m_debugger)
    delete m_debugger, m_debugger  = 0;

  if (m_debuggee)
    delete m_debuggee, m_debuggee  = 0;
 
}
//=====================================
void GRLauncher::run()
{
  if (!m_debugMode && m_debuggee)
  {
     m_debuggee -> setWD(m_wd);
     m_debuggee->load(m_dataFName);
  }
}

//=====================================
void GRLauncher::stop()
{
  if (m_debuggee)
    m_debuggee->stop();
}
//=====================================
void GRLauncher::close()
{
  if (m_debuggee)
    GReATDebuggerFactory::m_thrd->join(); 
  if (m_front)
    GRDI::GRDFrontEndFactory::m_thrd->join(); 
}