
#include <sstream>
#include "GReATDebugger.h"
#include "GReATDebuggerVars.h"
#include "../GRDBackEnd/IGRDBackend.h"
#include "GReATParametersFactory.h"
#include "GReATFactory.h"

#include <fstream>

//=========================
GReATDebugger::GReATDebugger():
                  m_back(0),
                  m_grParams(0),
                  m_loaded(false),
                  m_init(false),
                  m_up (false),
                  m_great(0),   
                  m_grcLoader(0),
                  m_debugMode(false),
                  m_quitNonDebug(false),
                  m_generalBackEnd(0),
                  m_progress(0)
{         
}
//================================
GReATDebugger::~GReATDebugger()
{
  m_back = 0;
  m_generalBackEnd = 0;
  if (m_grcLoader)
    delete m_grcLoader, m_grcLoader = 0;


  if (m_great)
    delete m_great, m_great = 0;

  clearSource();
  if (m_grParams)
    delete m_grParams, m_grParams = 0;
}

//================================

GRDI::FunctionDescr&  GReATDebugger::regFunction(const GR::RuleBase& rule,
                                                 bool& exists )
{
  RuleFunctionDescrMap::iterator it = m_sourceMap.find(rule);
  if (it == m_sourceMap.end())
  {


    m_sourceMap[rule].m_func = GRVariableFactory::createFunction(rule);

    const std::string& a = rule.position();
    std::string b =a.substr(1,a.size()-2);
    m_sourceMap[rule].m_xPos = atoi(b.substr(0,b.find(",")).c_str()); 

    exists = false;
    return m_sourceMap[rule];
  }
  exists = true;
  return it->second;
}

//================================
GRDI::FunctionDescr&  GReATDebugger::findFunction(const GR::RuleBase& rule)
{
  
  RuleFunctionDescrMap::iterator it = m_sourceMap.find(rule);
  if (it == m_sourceMap.end())
  {

    m_sourceMap[rule].m_func = GRVariableFactory::createFunction(rule);

    Udm::Object obj = Udm::Object::Cast( rule );
    throw std::invalid_argument("can't find function "+ std::string(rule.name()));
  }
  return it->second;
}
//================================
GRDI::IVariablePtr&  GReATDebugger::findVariable(const GR::ObjectWrapper& var)
{
  
  ObjectWrapperIVariablePtrMap::iterator it = m_varMap.find(var);
  if (it == m_varMap.end())
  {
    std::string name;
    getObjPath(var, name);
    throw std::invalid_argument("can't find variable " + name);
  }
  return it->second;
}

//================================
GRDI::IVariablePtr&  GReATDebugger::regVariable(const GR::ObjectWrapper& var)
{
  ObjectWrapperIVariablePtrMap::iterator it = m_varMap.find(var);
  if (it == m_varMap.end())
    m_varMap[var]= GRVariableFactory::createVariable(var);
  else
    throw std::invalid_argument("varibale duplication");

  return m_varMap[var];
}
//================================
void GReATDebugger::addVariables(const GR::RewritingRule& rule,
                                       GRDI::FunctionDescr& descr)
{
  const set< GR::ObjectWrapper>& os = rule.ObjectWrapper_kind_children();
  set< GR::ObjectWrapper>::const_iterator it = os.begin();
	for(; it!= os.end(); ++it)
	{
   const GR::ObjectWrapper& po = *it;

   GRDI::IVariablePtr&  newVar = regVariable(po);
   GRDI::IVariableImpl::Type type = GRDI::IVariableImpl::Type::PATTERN;

   const string& rt = rule.ruleType();
   if ((rt == "BLOCK") || (rt == "TEST"))
   {
     const string& prt = po.type().name();
     if( prt == "InputPort")
       type = GRDI::IVariableImpl::Type::INPUT;
     else
     if( prt == "OutputPort")
       type = GRDI::IVariableImpl::Type::OUTPUT;
   }
   else
   {
     const PassAlongSet& spa = po.srcObjectWrapper();
     const PassAlongSet& dpa = po.dstObjectWrapper();

     if (!spa.empty() && !dpa.empty())
       type = GRDI::IVariableImpl::Type::IO;
     else
     if (!spa.empty() && dpa.empty())
       type = GRDI::IVariableImpl::Type::INPUT;
     else
     if (spa.empty() && !dpa.empty())
       type = GRDI::IVariableImpl::Type::OUTPUT;
   }
   newVar->getIVariableImpl()->m_type = type;
   descr.m_varTable[newVar];
   
  }
}
//================================
GRDI::FunctionDescr&  GReATDebugger::addFunction(const GR::RuleBase& rule)
{
  bool exists = false;
  GRDI::FunctionDescr& curr = regFunction(rule, exists);

  if (exists || ((const std::string&)(rule.type().name()) == "RewritingRuleProxy"))
    return curr;

  const GR::RewritingRule& rr = GR::RewritingRule::Cast( rule );

  const set< GR::RuleBase>& crs = rr.RuleBase_kind_children();
  set< GR::RuleBase>::const_iterator itRules = crs.begin();

	for(; itRules!= crs.end(); ++itRules)
	{
    const GR::RuleBase& childRule = *itRules;

    GRDI::FunctionDescr& currChild  = addFunction(childRule);
    curr.m_calls.push_back(currChild.m_func);
//x    std::cout << curr.m_func->print() << ": ";
//x    std::cout << currChild.m_func->print() << std::endl;
  }
  
  addVariables(rr, curr);
  return curr;
}

//================================
void GReATDebugger::buildFuncDependencies(const GR::RuleBase& rule)
{
  if ((std::string(rule.type().name()) == "RewritingRuleProxy"))
    return;

  const GR::RewritingRule& rr = GR::RewritingRule::Cast( rule );
  const set< GR::RuleBase>& crs = rr.RuleBase_kind_children();


  set< GR::RuleBase>::const_iterator itRules = crs.begin();
  for(; itRules!= crs.end(); ++itRules)
	{
    const GR::RuleBase& childRule = *itRules;

    GRDI::FunctionDescr& childFuncDescr = findFunction(childRule);

    set<GR::Sequence> ssr = childRule.srcRule();
	  for(set<GR::Sequence>::iterator srIt = ssr.begin(); srIt!=ssr.end(); srIt++)
	  {	
		  const GR::Sequence& seq = *srIt;

      const GR::RuleBase& srcRule = seq.srcRule_end();
      const GRDI::FunctionDescr& srcFunc = findFunction(srcRule);

			if((GR::RuleBase::Cast(srcRule.RewritingRule_parent()) != rule) 
        ||         (srcRule  == childRule) 
         )
			continue;


      childFuncDescr.m_dependsOn.push_back(srcFunc.m_func);

    }
  }

}

//========================================
void GReATDebugger::buildSource()
{
  
  GR::RootFolder      rootFolder;
  m_grParams->getRootFolder(rootFolder);
 	

  clearSource();

  const set< GR::RewritingRule>& topRules = rootFolder.RewritingRule_kind_children();

  set< GR::RewritingRule>::const_iterator itS = topRules.begin();
  for(;itS != topRules.end(); ++itS)
  {
    const GR::RuleBase& rule = *itS;
    addFunction(rule);
    m_back->addRootFunction(findFunction(rule).m_func);
  }

  GR::RewritingRule rule;
  m_grParams->getStartRule(rule);

  m_back->setStartFunction(findFunction(rule).m_func);


  RuleFunctionDescrMap::const_iterator itD = m_sourceMap.begin();
  for (;itD != m_sourceMap.end(); ++itD)
    buildFuncDependencies(itD->first);

  RuleFunctionDescrMap::iterator itR = m_sourceMap.begin();
  for (;itR != m_sourceMap.end(); ++itR)
  {
    const GR::RuleBase& currRule = itR->first;
    if ((const std::string&)(currRule.type().name()) == "RewritingRuleProxy")
    {
  	  const GR::RuleBase& ref = GR::RewritingRuleProxy::Cast(currRule).refersTo();
      const GRDI::FunctionDescr& funcDescr = findFunction(ref);
      itR->second.m_ref = funcDescr.m_func; 
    }
  }

  RuleFunctionDescrMap::const_iterator it = m_sourceMap.begin();
  for (;it != m_sourceMap.end(); ++it)
    m_back->registerFunctionCall(it->second);

}
//=========================================
void GReATDebugger::clearSource()
{
  if (m_back)
    m_back->clearSource();
  m_sourceMap.clear();
  m_varMap.clear();
}

//=========================================
void GReATDebugger::addVarValue(GRDI::FunctionCallDescr& funcCallDescr,
                                const GR::RuleBase& rule,
                                const PassAlongPackets& arguments)
{
  if (arguments.size())
  {

    GRDI::VariableValueMap& varMap = funcCallDescr.m_varValMap;

    PassAlongPackets::const_iterator it = arguments.begin();
    for(;it != arguments.end(); ++it)
    {
      const PassAlongPacket& packet = *it;
      PassAlongPacket::const_iterator itm = packet.begin();
      for (; itm != packet.end(); ++itm)
      {

        const GRDI::IVariablePtr& var = findVariable(itm->first);
        GRDI::VariableValueMap::iterator itV = varMap.find(var) ;
      
        if (itV == varMap.end())
        {
          throw std::invalid_argument("can't set value of variable " + var->name());
        }

        GRDI::IVariableValuePtr& varValue = itV->second;
        if (varValue == GRDI::IVariableValuePtr())
          varValue = GRVariableFactory::createVariableValue(GRVariableValue());

        GRDI::IVariableValueImpl& valImpl =  *(varValue->getIVariableValueImpl());
        valImpl.addValue(itm->second);
      }
    }
  }
}
//=========================================
void GReATDebugger::addMatchVarValue(GRDI::FunctionCallDescr& funcCallDescr,
                                   const MatchVector& match)
{
  if (match.size())
  {
    //std::ofstream lout("lofax");

    GRDI::VariableValueMap& varMap = funcCallDescr.m_varValMap;

    MatchVector::const_iterator it = match.begin();
    for(; it != match.end(); ++it)
    {
      const map<GR::ObjectWrapper, Object>& oom =  it->grObj2BoundObj;
  	  map<GR::ObjectWrapper, Object>::const_iterator itM = oom.begin();
      for(;	itM != oom.end(); ++itM)
 	    {

        //const GR::ObjectWrapper& grObj = GR::ObjectWrapper::Cast(grTransformation->ObjectById(itM->first));
        const GR::ObjectWrapper& grObj = itM->first;

        if ((const std::string&)grObj.action() == "Bound")
        {

          const GRDI::IVariablePtr& var = findVariable(grObj);
          GRDI::VariableValueMap::iterator itV = varMap.find(var) ;

          if (itV == varMap.end())
             throw std::invalid_argument("can't set value of pattern " + var->name());

          if (itV->first->getIVariableImpl()->m_type == GRDI::IVariableImpl::PATTERN)
          {

            GRDI::IVariableValuePtr& varValue = itV->second;
            if (varValue == GRDI::IVariableValuePtr())
              varValue = GRVariableFactory::createVariableValue(GRVariableValue());

            GRDI::IVariableValueImpl& valImpl =  *(varValue->getIVariableValueImpl());
        
//             std::string n1;
//             getObjPath(grObj, n1);

       	 //   const std::string& paradigmName = grObj.paradigmName();
       //     Udm::DataNetwork* hostdn = m_generalBackEnd->GetBackend(paradigmName);
            if (grObj.dataNetworkID() != 0)
            {
		        	Udm::DataNetwork* hostdn = Udm::DataNetwork::GetDnById(grObj.dataNetworkID());
              const Object& bo = itM->second;
           //   if (bo.uniqueId()!=0)
              {
			          const Object& obj = hostdn->ObjectById(bo.uniqueId());
                valImpl.addValue(obj);
              }
            }
  //             std::string n2;
//             getObjPath(obj, n2);
//             lout << n1 << "->" << n2<< std::endl;

          }
        }
      }
    }
  }
}

 
