#pragma warning( disable : 4503)

#include "StdAfx.h"

#include "GRThread.h"
#include "GREngineDlgDlg.h"
#include "GReATFactory.h"
#include "GReATParametersFactory.h"
#include "GRCC.h"
#include "GRCNames.h"
#include "GRCLoader.h"
#include "GReATConfig.h"
#include "GReATConfigMeta.h"
#include "GReATParameters.h"
#include "GReATUtils.h"
#include <algorithm>

//============================
void Notify::reportProgress(double prg) 
{
	if (!m_quit)
	{
		UINT nprg =  (UINT )(m_maxPrg * prg);
		// FIXME KMS Should this be PostMessage?
		m_dlg.SendMessage(SET_PROGRESS, (LPARAM) nprg);
	}
}

void Notify::reportExcuteRule(const string &rule)
{
	if (!m_quit)
	{
		CString str(rule.c_str());
		m_dlg.SendMessage(SET_EXECUTERULE, (LPARAM)(LPCTSTR) str);
	}
}

//============================
void Notify::finished() 
{
	m_dlg.PostMessage(GR_FINISHED, 0);
}

//============================
GRThread::GRThread(const std::string& fname) :
				m_fname(fname),
				m_notify(NULL)
{
}

GRThread::GRThread( const std::string& fname,
					Notify* notify) :
                 m_fname(fname),
				 m_notify(notify)
                  
{
}

GRThread::GRThread(const std::string& fname, 
				   const vector<std::string> &inputs,
				   Notify* notify) :
				m_fname(fname),
				m_inputs(inputs),
				m_notify(notify)
{
}

//============================
void GRThread::operator()()
{
	CoInitialize(0);  //Magic happens here
	IGReATParameters* grParams  = 0;
	IGReAT* great = 0;
//	  ICINTCodeGatherer* cintDll = 0;

	try
	{
		grParams = GReATParametersFactory::create();
		if(m_inputs.empty())  
			grParams->open(m_fname);
		else 
			grParams->open(m_fname, m_inputs);

		GeneralBackendPtr dataNetworkMap;
		grParams->getGeneralBackend(dataNetworkMap);

		UdmDataNetworkPtr transformation;
		grParams->getTransformation(transformation);

		great = GReATFactory::create();
		great->setCtrlNotify(m_notify);
		great->init(dataNetworkMap, transformation);

/////////////////////////////////////////////////////////////
	GReATConfig::RootFolder cfgRoot= GReATConfig::RootFolder::Cast( ((GReATParameters*)grParams)->sdnConfig->GetRootObject());
	std::set< GReATConfig::MetaInformation> mis= cfgRoot.MetaInformation_kind_children();	
	GReATConfigMeta metaInfo( *mis.begin());
	std::string configPath= getPath( m_fname);

	CCodeGenConfig cc_cfg;
	cc_cfg._transFileName= getFullPath( configPath, metaInfo.getTransformationFileName());
	cc_cfg._depFileName= cc_cfg._transFileName;
	cc_cfg._udmDirName= getPath( getFullPath( configPath, metaInfo.getUdmProjectFileName()), false);
	//
	typedef std::map< std::string, const Udm::DataNetwork*> BackendMap_t;
	BackendMap_t dns;
	metaInfo.getGeneralBackend( dns, configPath);
	StrList_t metanames;
	getFirst( std::make_pair( std::string(), (const Udm::DataNetwork*) 0));	// template instantiation to satisfy the linker
	std::transform( dns.begin(), dns.end(), std::back_inserter( metanames), getFirst< std::string, const Udm::DataNetwork*>);
	if ( metaInfo.isUdmProjectFile()) {
		metanames.push_back( metaInfo.getCrossLinkDiagramName());
	}
	cc_cfg._metas= metanames;
	//
    #ifdef _DEBUG
      cc_cfg._debug = true;
    #else
      cc_cfg._debug = false;
    #endif
	cc_cfg._dynamic= true;
	//
	GRCC grcc( cc_cfg);
	grcc.generateLibrary();
	//
	std::string dll_name= getGreatTempDir() + GRCNames::generateDllName( dropPath( dropExtension( cc_cfg._transFileName)),cc_cfg._debug);
	GRCLoader grcLoader( dll_name, grParams->getMetas());
    HINSTANCE hDll  = grcLoader.loadDll();
    great->setCINTDll(hDll);
/////////////////////////////////////////////////////////////
	/*
  	cintDll = CINTCodeGathererFactory::create();
    UserCodeGenConfig cfg;
    cfg.m_depFileName = grParams->getTransfLastWrittenFileName();
    cfg.m_udmDirName = grParams->getUdmDirName();
    cfg.m_transf = transformation;
    cfg.m_metas = grParams->getMetas();

//    StrVec incFiles, libFiles, incDirs, libDirs;
//    grParams->getUserCodeInfo(incFiles, libFiles, incDirs, libDirs);

//    cfg.m_incFiles = incFiles;
//    cfg.m_libFiles = libFiles;
//    cfg.m_incDirs = incDirs;
//    cfg.m_libDirs = libDirs;

    #ifdef _DEBUG
      cfg.m_debug = true;
    #else
      cfg.m_debug = false;
    #endif

    cintDll->generateDll(cfg); 
    HINSTANCE hDll  = cintDll->loadDll();
    great->setCINTDll(hDll);
	*/
/////////////////////////////////////////////////////////////

		GR::RewritingRule startRule;
		PassAlongPackets arguments;
		grParams->getStartRule(startRule);
		grParams->getPassAlongPackets(arguments);
		great->Execute(startRule, arguments);
		//====================
		arguments.clear();
		if (great)
			delete great, great = 0;
		if(!m_notify->m_cancel)
		{
			grParams->close();
			grParams->getOutputMsg(CGREngineDlgDlg::outputMsg);
		}
		else grParams->closeNoSave();

		if (grParams)
			delete grParams, grParams = 0;
		grcLoader.closeDll();
	}
	catch(udm_exception &exc)
	{
		AfxMessageBox(CString("Exception: ") + CString(exc.what()));
		if (great)
			delete great, great = 0;
		grParams->closeNoSave();
		m_notify->m_quit = true;

		if (grParams)
		delete grParams, grParams = 0;

		if(m_notify)  m_notify->finished();

//		if (cintDll)
//		delete cintDll, cintDll = 0;
//		grcLoader.closeDll();

		CoUninitialize();

		// FIXME: GRT-81 throwing this causes the program to terminate
		// throw exc;

		// FIXME: we need to call grcLoader.closeDll(); in some cases
	}

//	if (grParams)
//		delete grParams, grParams = 0;

	if(m_notify)  m_notify->finished();

//	  if (cintDll)
//    delete cintDll, cintDll = 0;
	
	CoUninitialize();
}
