/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/


// UdmApp.cpp: implementation of the CUdmApp class.
// This file was automatically generated as UdmApp.cpp
// by UDM Interpreter Wizard on Monday, May 13, 2002 13:45:42

// Tihamr Levendovszky 05-13-02

#include "stdafx.h"
#include <afxdlgs.h> // For CFileDialog
#include "resource.h"

//#include <iostream.h>
#include <atlbase.h>
#include <comdef.h>

#include <time.h>
#include "UdmStatic.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UmlExt.h"
#include "UdmUtil.h"

#include "UdmApp.h"
#include "UdmConfig.h"

#include "UMLModelTransformer.h"

#include "umt_exception.h"

#include "FileInfoDialog.h"

#define GREAT_SILENT_MODE 128

extern void dummy(void); // Dummy function for UDM meta initialization

char* GetRelativeFilename(const char *currentDirectory,const char *absoluteFilename);

/*********************************************************************************/
/* Initialization function. The framework calls it before preparing the backend. */
/* Initialize here the settings in the config static object.					 */
/* Return 0 if successful.														 */
/*********************************************************************************/
int CUdmApp::Initialize()
{
	// TODO: Your initialization code comes here...
	return 0;
}



/* 
Remarks to CUdmApp::UdmMain(...):
0.	The p_backend points to an already open backend, and the framework 
	closes it automatically. DO NOT OPEN OR CLOSE IT!
	To commit changes use p_backend->CommitEditSequence().
	To abort changes use p_backend->AbortEditSequence().
	To save changes to a different file use p_backend->SaveAs() or p_backend->CloseAs().

1.	Focus is the currently open model.

2.	The possible values for param (from GME DecoratorLib.h component_startmode_enum):
	GME_MAIN_START			=   0,
	GME_BROWSER_START		=   1,
	GME_CONTEXT_START		=   2,
	GME_EMBEDDED_START		=   3,
	GME_MENU_START			=  16,
	GME_BGCONTEXT_START		=  18,
	GME_ICON_START			=  32,
	METAMODEL_CHECK_SYNTAX	= 101

    GREAT_SILENT_MODE		= 128   I couldn't find DecoratorLib.h at all, but 128 is 
	                                the silent mode parameter for the Meta Interpreter
								    so that's what I used here - Brian

 3. The framework catches all the exceptions and reports the error in a message box,
	clean up and close the transactions aborting the changes. You can override this 
	behavior by catching udm_exception. Use udm_exception::what() to form an error 
	message.
*/

/***********************************************/
/* Main entry point for Udm-based Interpreter  */
/***********************************************/

BOOL CUdmApp::runGRE;
BOOL CUdmApp::runCG; 
BOOL CUdmApp::genCFG;
BOOL CUdmApp::genGR;
BOOL CUdmApp::genUDM;
string CUdmApp::mgaDir;
string CUdmApp::configName;
string CUdmApp::grName;
string CUdmApp::udmName;

void CUdmApp::UdmMain(
					 Udm::DataNetwork* p_backend,		// Backend pointer(already open!)
					 Udm::Object focusObject,			// Focus object
					 set<Udm::Object> selectedObjects,	// Selected objects
					 long param)						// Parameters
{	
		
		UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
		UMLModelTransformer::Configurations cfold;
		UMLModelTransformer::Configuration cnf;
		vector<UMLModelTransformer::Configuration> confs;
		
		// Checking to see the number of Package Folders
		string udmExt;
		string udmPackageName;
		vector<UMLModelTransformer::Package> packs = rf.Package_children();
		vector<UMLModelTransformer::RootFolder> prfs = rf.RootFolder_children();
		int sz = packs.size() + prfs.size();
		if(sz == 0)
		{	throw udm_exception("File Info Error: Couldn't find any UML Packages. Please add at least one UML package");
		}
		else if(sz == 1)
		{	udmExt = ".xml";
			if(packs.size() == 1)
				udmPackageName = (*(packs.begin())).name();
			else
				udmPackageName = (*(prfs.begin())).name();
		}
		else
		{	udmPackageName = rf.name();
			udmExt = ".udm";
		}
		
		// Get the configuration model
		if(focusObject != Udm::null && (string)focusObject.type().name() == "Configuration")
		{	cnf = UMLModelTransformer::Configuration::Cast(focusObject);
		}
		else
		{	vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
			for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
			{	cfold = *i;
				vector<UMLModelTransformer::Configuration> cfgs = cfold.Configuration_kind_children();
				confs.insert(confs.end(), cfgs.begin(), cfgs.end());
			}
			if(confs.size() == 0)
			{	if(cfold == Udm::null)
				{	cfold = UMLModelTransformer::Configurations::Create(rf);
					cfold.name() = "zz_Config";
				}
				cnf = UMLModelTransformer::Configuration::Create(cfold);
				cnf.name() = "Config";

			}
			else if(confs.size() > 1)
			{	throw udm_exception("File Info Error: More than one configuration found.\nEither call the interpretation from the desired configuration or delete the others.");
			}
			else 
			{	cnf = *(confs.begin());
			}
		}
		
		// Checking for correct config file
		UMLModelTransformer::MetaInformation mi;
		vector<UMLModelTransformer::MetaInformation> mis = cnf.MetaInformation_kind_children();
		if(mis.size() > 1)
		{	//Error: The configuration file doesn't contain a meta information
			throw udm_exception("File Info Error: The configuration model comtains more than one Meta info atom.\nPlease delete all other Meta Info atoms.");
		}
		else if(mis.size() == 0)
		{	// Create a Meta info atom
			mi = UMLModelTransformer::MetaInformation::Create(cnf);
			mi.name() = "";
			mi.position() = "(100, 10)";

		}
		else
		{	mi = *(mis.begin());
		}
		
		configName = cnf.ConfigFile();
		grName = mi.TransformationFile();
		udmName = mi.UdmProjectFile();
		string genName = cnf.CodeGenFileName();

		if(param == GREAT_SILENT_MODE || configName == "")
		{
			configName = CUdmApp::mgaDir + (string)cnf.name() + ".mga";
			if(((string)cnf.ConfigFile()).empty()) cnf.ConfigFile() = configName;
		}

		if(grName == "")
			grName = (string)rf.name() + "-gr.xml";

		if(udmName == "")
			udmName = "Udm\\" + udmPackageName + udmExt;

		if(genName == "")
			genName = "Gen\\Gen" + (string)rf.name();

		// Calling the FileInfoDialog
		if(param != GREAT_SILENT_MODE)
		{
			CFileInfoDialog fileInfoDlg;
		
			fileInfoDlg.m_ConfigName = configName.c_str();
			fileInfoDlg.m_GrName     = grName.c_str();
			fileInfoDlg.m_UdmName    = udmName.c_str();
			fileInfoDlg.m_GenName    = genName.c_str();
			fileInfoDlg.m_UdmExt     = udmExt.c_str();
			fileInfoDlg.m_MgaDir     = CUdmApp::mgaDir.c_str();

//			if(fileInfoDlg.DoModal() != IDOK)
//				throw umt_exception("File Dialog Canceled");

			if(fileInfoDlg.DoModal() == IDCANCEL)
				return;

			configName = (LPCSTR)fileInfoDlg.m_ConfigName;
			grName     = (LPCSTR)fileInfoDlg.m_GrName;
			udmName    = (LPCSTR)fileInfoDlg.m_UdmName;
			genName    = (LPCSTR)fileInfoDlg.m_GenName;

			CUdmApp::runGRE = fileInfoDlg.m_RunGRE;
			CUdmApp::runCG  = fileInfoDlg.m_RunCG;
			CUdmApp::genCFG = fileInfoDlg.m_GenCFG;
			CUdmApp::genGR = fileInfoDlg.m_GenGR;
			CUdmApp::genUDM = fileInfoDlg.m_GenUDM;
			//////////////////////////////////
		}
		
		else //Silent Mode
		{
			//leave all those strings at the default

			//we don't want to run GRE or CG
			CUdmApp::runGRE = false;
			CUdmApp::runCG = false;
			CUdmApp::genCFG = true;
			CUdmApp::genGR = true;
			CUdmApp::genUDM = true;

			//the Silent Mode param will be passed on to the other 
			//interpreters when they are called in RawComponent.cpp
			//only GenerateConfig needs it
		}
		cnf.ConfigFile() = configName;
		mi.TransformationFile() = grName;
		mi.UdmProjectFile() = udmName;
		cnf.CodeGenFileName() = genName;
}



#ifdef _DEBUG
/*****************************************************/
/* Debug time helper function. If the object has an  */
/* attribute called "name", this function retreives  */
/* it to help you to find it in the model during the */
/* application development.	Usualy every GME Object	 */
/* has a "name" attribute. If an object hapens not	 */
/* to have it,function retreives <no name specified>.*/
/*****************************************************/
string CUdmApp::ExtractName(Udm::Object ob)
{
	Uml::Class cls= ob.type();				
	set<Uml::Attribute> attrs=cls.attributes();		
	
	// Adding parent attributes
	set<Uml::Attribute> aattrs=Uml::AncestorAttributes(cls);
	attrs.insert(aattrs.begin(),aattrs.end());

	for(set<Uml::Attribute>::iterator ai = attrs.begin();ai != attrs.end(); ai++) 
	{
		if(string(ai->type())=="String")
		{
			string str=ai->name();
			if(str=="name")
			{
				string value=ob.getStringAttr(*ai);
				if(value.empty())value="<empty string>";
				return value;
			}			
		}				
	}	
	return string("<no name specified>");
}
#endif
