/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/


// UdmApp.cpp: implementation of the CUdmApp class.
// This file was automatically generated as UdmApp.cpp
// by UDM Interpreter Wizard on Monday, May 13, 2002 13:45:42

// MFC includes
#include "stdafx.h"
#include <afxdlgs.h> // For CFileDialog
#include "resource.h"
// windows headers
#include <windows.h>	// for cursors
#include <atlbase.h>
#include <comdef.h>
// C/C++ standard headers
#include <time.h>
#include <direct.h> // for _mkdir()
#include <fstream>
// GME interpreter headers
#include "UdmApp.h"
#include "UdmConfig.h"
// UDM generic headers
#include "UdmStatic.h"
#include "UdmDom.h"
#include "UdmGME.h"
#include "UmlExt.h"
#include "UdmUtil.h"
// UDM domain-specific API
#include "UMLModelTransformer.h"
// GReAT includes
#include "GReATUtils.h"

extern void dummy(void); // Dummy function for UDM meta initialization

/*********************************************************************************/
/* Initialization function. The framework calls it before preparing the backend. */
/* Initialize here the settings in the config static object.					 */
/* Return 0 if successful.														 */
/*********************************************************************************/
int CUdmApp::Initialize()
{
	// TODO: Your initialization code comes here...
	return 0;
}



/* 
Remarks to CUdmApp::UdmMain(...):
0.	The p_backend points to an already open backend, and the framework 
	closes it automatically. DO NOT OPEN OR CLOSE IT!
	To commit changes use p_backend->CommitEditSequence().
	To abort changes use p_backend->AbortEditSequence().
	To save changes to a different file use p_backend->SaveAs() or p_backend->CloseAs().

1.	Focus is the currently open model.

2.	The possible values for param (from GME DecoratorLib.h component_startmode_enum):
	GME_MAIN_START			=   0,
	GME_BROWSER_START		=   1,
	GME_CONTEXT_START		=   2,
	GME_EMBEDDED_START		=   3,
	GME_MENU_START			=  16,
	GME_BGCONTEXT_START		=  18,
	GME_ICON_START			=  32,
	METAMODEL_CHECK_SYNTAX	= 101

 3. The framework catches all the exceptions and reports the error in a message box,
	clean up and close the transactions aborting the changes. You can override this 
	behavior by catching udm_exception. Use udm_exception::what() to form an error 
	message.
*/

///////////////////////////////////////////////////////////////////////////////
std::string getTransformationFileName( const UMLModelTransformer::RootFolder& rf, const Udm::Object& focus) 
{
	UMLModelTransformer::Configuration cnf;
	vector<UMLModelTransformer::Configuration> confs;
	// Get the configuration model
	if( focus != Udm::null && (string)focus.type().name() == "Configuration")
	{	cnf = UMLModelTransformer::Configuration::Cast(focus);
	}
	else
	{	vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
		for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
		{	UMLModelTransformer::Configurations cfold = *i;
			vector<UMLModelTransformer::Configuration> cfgs = cfold.Configuration_kind_children();
			confs.insert(confs.end(), cfgs.begin(), cfgs.end());
		}
		if(confs.size() == 0)
		{	throw udm_exception("Error: Couldnt find any configuration file");
		}
		if(confs.size() > 1)
		{	throw udm_exception("Error: More than one configuration found. Please make the desired one the focused object and then press interepret");
		}
		cnf = *confs.begin();
	}
	vector< UMLModelTransformer::MetaInformation> mis= cnf.MetaInformation_kind_children();
	UMLModelTransformer::MetaInformation mi= *mis.begin();
	return mi.TransformationFile();
}

///////////////////////////////////////////////////////////////////////////////
class CmpFuncProcessor {
public:
	CmpFuncProcessor( std::ostream& os) : _os( os) {}
	void operator()( const UMLModelTransformer::CodeLibrary& cl) {
		std::vector< UMLModelTransformer::CompareFunction> cmpfuncs= cl.CompareFunction_kind_children();
		for_each_ref( cmpfuncs.begin(), cmpfuncs.end(), *this);
	}
	void operator()( const UMLModelTransformer::CompareFunction& cf) {
		// compare function template
		_os << "template <class T>" << endl;
		_os << "bool " << ( string)cf.name() << "( const T& " << ( string) cf.LHS() << ", const T& " << ( string) cf.RHS() << ") {" << endl;
		_os << "\t" << ( string) cf.FuncBody() << endl << "}" << endl << endl;
		// compare function caster template
		_os << "template <class BASE, class T>" << endl;
		_os << "bool " << ( string)cf.name() << "_caster( const BASE& " << ( string) cf.LHS() << ", const BASE& " << ( string) cf.RHS() << ") {" << endl;
		std::string lhsName= ( string) cf.LHS()+ "_casted";
		std::string rhsName= ( string) cf.RHS()+ "_casted";
		_os << "\tT " << lhsName << "= T::Cast( " << ( string) cf.LHS() << ");" << endl;
		_os << "\tT " << rhsName << "= T::Cast( " << ( string) cf.RHS() << ");" << endl;
		_os << "\t" << "return " << ( string)cf.name() << "( " << lhsName << ", " << rhsName << ");" << endl;
		_os << "}" << endl << endl;
	}
private:
	std::ostream& _os;
};

static void open_ofstream(std::string filename, std::ofstream& ofs)
{
	ofs.open(filename.c_str());
	if (ofs.fail())
	{
		char cTemp[256];
		DWORD dTemp = GetLastError();
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, dTemp, 0, cTemp, 256, NULL);
		throw udm_exception("Can't open '" + filename + "' for writing:" + cTemp);
	}
}

///////////////////////////////////////////////////////////////////////////////
// generates <filenamebase>_cmptempl.h
void generateCompareFunctions( const std::vector< UMLModelTransformer::CodeLibrary>& codeLibs, const std::string& filenamebase)
{
	// create file
	std::string GreatTempPath= getGreatTempDir();
	std::ofstream cmptempl_h;
	open_ofstream(GreatTempPath + filenamebase + "_cmptempl.h", cmptempl_h);
	// print #include code
	cmptempl_h << "#include \"" << filenamebase << "_userinc.h\"" << endl << endl; // user code library includes
	// print compare function code
	CmpFuncProcessor cmpFuncProc( cmptempl_h);
	for_each_ref( codeLibs.begin(), codeLibs.end(), cmpFuncProc);
	// close file
	cmptempl_h.close();
}

std::string processIncFile( const std::string& incFile) {
	int len= incFile.length();
	if ( incFile[ 0] == '<') {
		// must end with '>'
		if ( incFile[ len-1] == '>') {
			return incFile;
		} else {
			return incFile + ">";
		}
	} else if ( incFile[ 0] == '\"') {
		// must end with '\"'
		if ( incFile[ len-1] == '\"') {
			return incFile;
		}  else {
			return incFile + "\"";
		}
	} else {
		// not enclosed in quotes nor brackets -> use quotes by default
		return "\"" + incFile + "\"";
	}
}

///////////////////////////////////////////////////////////////////////////////
class UserIncludeProcessor {
public:
	UserIncludeProcessor( std::ostream& os) : _os( os) {}
	void operator()( const UMLModelTransformer::CodeLibrary& cl) {
		std::vector< UMLModelTransformer::UserCodeLibrary> ucls= cl.UserCodeLibrary_kind_children();
		for_each_ref( ucls.begin(), ucls.end(), *this);
	}
	void operator()( const UMLModelTransformer::UserCodeLibrary& ucl) {
		std::vector< std::string> tokenized;
		int numOfTokens= tokenize_csv_lines( (string) ucl.IncludeFiles(), tokenized);
		for_each_ref( tokenized.begin(), tokenized.end(), *this);
	}
	void operator()( const std::string& incFile) {
		_os << "#include " << processIncFile( incFile) << endl;
	}
private:
	std::ostream& _os;
};

///////////////////////////////////////////////////////////////////////////////
// generates <filenamebase>_userinc.h
void generateUserIncludeHeader( const std::vector< UMLModelTransformer::CodeLibrary>& codeLibs, const std::string& filenamebase)
{
	// create file
	std::string GreatTempPath= getGreatTempDir();
	std::ofstream userinc_h;
	open_ofstream( (GreatTempPath+ filenamebase+ "_userinc.h").c_str(), userinc_h);
	// print user include files
	UserIncludeProcessor userIncludeProc( userinc_h);
	for_each_ref( codeLibs.begin(), codeLibs.end(), userIncludeProc);
	// close file
	userinc_h.close();
}

///////////////////////////////////////////////////////////////////////////////
class UserIncPathProccesor {
public:
	UserIncPathProccesor( const std::string& cfgFilePath, std::ostream& os) : _cfgFilePath( cfgFilePath), _os( os) {}
	void operator()( const UMLModelTransformer::CodeLibrary& cl) {
		std::vector< UMLModelTransformer::UserCodeLibrary> ucls= cl.UserCodeLibrary_kind_children();
		for_each_ref( ucls.begin(), ucls.end(), *this);
	}
	void operator()( const UMLModelTransformer::UserCodeLibrary& ucl) {
		std::vector< std::string> tokenized;
		int numOfTokens= tokenize_csv_lines( (string) ucl.IncludePath(), tokenized);
		for_each_ref( tokenized.begin(), tokenized.end(), *this);
	}
	void operator()( const std::string& incPath) {
		_os << " " << encloseinQuotes( getFullPath( _cfgFilePath, stripTrailingSlash( stripQuotes( incPath))));
	}
private:
	const std::string& _cfgFilePath;
	std::ostream& _os;
};

///////////////////////////////////////////////////////////////////////////////
// generates <filenamebase>_codeinc.mpb
void generateIncludeMpc( const std::vector< UMLModelTransformer::CodeLibrary>& codeLibs, const std::string&filenamebase, const std::string& cfgFilePath) {
	// create file
	std::string GreatTempPath= getGreatTempDir();
	std::ofstream inc_mpc;
	open_ofstream( (GreatTempPath+ filenamebase+ "_codeinc.mpb").c_str(), inc_mpc);
	// print mpc header 
	inc_mpc << "// MPC file to generate additional user code include directories" << endl;
	inc_mpc << "project {" << endl;
	inc_mpc << "\tincludes+=";
	// print user include paths
	UserIncPathProccesor userIncPathProc( cfgFilePath, inc_mpc);
	for_each_ref( codeLibs.begin(), codeLibs.end(), userIncPathProc);
	// print mpc footer
	inc_mpc << endl << "}" << endl;
	// close file
	inc_mpc.close();
}

///////////////////////////////////////////////////////////////////////////////
class UserLibraryProcessor {
public:
	UserLibraryProcessor( const std::string& cfgFilePath, std::string& libs, std::string& libPaths) : _cfgFilePath( cfgFilePath), _libs( libs), _libPaths( libPaths) {}
	void operator()( const UMLModelTransformer::CodeLibrary& cl) {
		std::vector< UMLModelTransformer::UserCodeLibrary> ucls= cl.UserCodeLibrary_kind_children();
		for_each_ref( ucls.begin(), ucls.end(), *this);
	}
	void operator()( const UMLModelTransformer::UserCodeLibrary& ucl) {
		std::vector< std::string> tokenized;
		// lib paths
		int numOfTokens= tokenize_csv_lines( (string) ucl.LibraryPath(), tokenized);
		for_each_ref( tokenized.begin(), tokenized.end(), *this);
		// libs
		tokenized.clear();
		tokenize_csv_lines( (string) ucl.LibraryFiles(), tokenized);
		for( std::vector< std::string>::const_iterator i= tokenized.begin(); i!= tokenized.end(); ++i) {
			_libs+= " " + encloseinQuotes( stripQuotes( *i));
		}
	}
	void operator()( const std::string& libPath) {
		_libPaths+= " " + encloseinQuotes( getFullPath( _cfgFilePath, stripTrailingSlash( stripQuotes( libPath))));
	}
private:
	const std::string& _cfgFilePath;
	std::string& _libs;
	std::string& _libPaths;
};

///////////////////////////////////////////////////////////////////////////////
// generates <filenamebase>_codelib.mpb
void generateLibraryMpc( const std::vector< UMLModelTransformer::CodeLibrary>& codeLibs, const std::string&filenamebase, const std::string& cfgFilePath){
	// create file
	std::string GreatTempPath= getGreatTempDir();
	std::ofstream lib_mpc;
	open_ofstream( (GreatTempPath+ filenamebase+ "_codelib.mpb").c_str(), lib_mpc);
	// print mpc header 
	lib_mpc << "// MPC file to generate user code library" << endl;
	lib_mpc << "project {" << endl;
	// print user libraries and lib paths
	std::string libs, libPaths;
	UserLibraryProcessor userLibProc( cfgFilePath, libs, libPaths);
	for_each_ref( codeLibs.begin(), codeLibs.end(), userLibProc);
	lib_mpc << "libs+=" << libs << endl;
	lib_mpc << "libpaths+=" << libPaths << endl;
	// print mpc footer
	lib_mpc << "}" << endl;
	// close file
	lib_mpc.close();
}

///////////////////////////////////////////////////////////////////////////////
std::string getConfigFile( Udm::DataNetwork* p_backend, Udm::Object focusObject) {
	UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
	UMLModelTransformer::Configuration cnf;
	vector<UMLModelTransformer::Configuration> confs;
	// Get the configuration model
	if(focusObject != Udm::null && (string)focusObject.type().name() == "Configuration")
	{	cnf = UMLModelTransformer::Configuration::Cast(focusObject);
	}
	else
	{	vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
		for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
		{	UMLModelTransformer::Configurations cfold = *i;
			vector<UMLModelTransformer::Configuration> cfgs = cfold.Configuration_kind_children();
			confs.insert(confs.end(), cfgs.begin(), cfgs.end());
		}
		if(confs.size() == 0)
		{	throw udm_exception("Config Error: Couldnt find any configuration model");
		}
		if(confs.size() > 1)
		{	throw udm_exception("Config Error: More than one configuration found. Please make the desired one the focused object and then press interepret");
		}
		cnf = *confs.begin();
	}
	string name = cnf.ConfigFile();
	if(name == "" || name[1] != ':')
	{	throw udm_exception("Config Error: No configuration file name. Run UMT Master interpreter.");
	}
	return name;
}

///////////////////////////////////////////////////////////////////////////////
void CUdmApp::UdmMain(
					 Udm::DataNetwork* p_backend,		// Backend pointer(already open!)
					 Udm::Object focusObject,			// Focus object
					 set<Udm::Object> selectedObjects,	// Selected objects
					 long param)						// Parameters
{	
 	try
 	{
		::SetCursor( ::LoadCursor( 0, IDC_WAIT));
		UMLModelTransformer::RootFolder rf = UMLModelTransformer::RootFolder::Cast(p_backend->GetRootObject());
		vector<UMLModelTransformer::CodeLibrary> codeLibs;
		// Get the code library models
		vector<UMLModelTransformer::Configurations> cfolds = rf.Configurations_children();
		for(vector<UMLModelTransformer::Configurations>::iterator i=cfolds.begin(); i!=cfolds.end();i++)
		{	UMLModelTransformer::Configurations cfold = *i;
			vector<UMLModelTransformer::CodeLibrary> cls = cfold.CodeLibrary_kind_children();
			codeLibs.insert(codeLibs.end(), cls.begin(), cls.end());
		}
//		if(codeLibs.size() == 0)
//		{	return; // no code library
//		}
		// generate c++ code for code library
		std::string filenamebase= dropPath( dropExtension( getTransformationFileName( rf, focusObject)));
		generateCompareFunctions( codeLibs, filenamebase);
		// generate user include header file
		generateUserIncludeHeader( codeLibs, filenamebase);
		// generate .mpc files
		std::string configFilePath= getPath( getConfigFile( p_backend, focusObject));
		generateIncludeMpc( codeLibs, filenamebase, configFilePath);
		generateLibraryMpc( codeLibs, filenamebase, configFilePath);

	}
	catch(udm_exception &e)
	{
		::SetCursor( ::LoadCursor( 0, IDC_ARROW));
		AfxMessageBox(e.what());
	    throw e; // letthe master interpreter know that something went wrong ... /kalmar
	}
	::SetCursor( ::LoadCursor( 0, IDC_ARROW));
}



#ifdef _DEBUG
/*****************************************************/
/* Debug time helper function. If the object has an  */
/* attribute called "name", this function retreives  */
/* it to help you to find it in the model during the */
/* application development.	Usualy every GME Object	 */
/* has a "name" attribute. If an object hapens not	 */
/* to have it,function retreives <no name specified>.*/
/*****************************************************/
string CUdmApp::ExtractName(Udm::Object ob)
{
	Uml::Class cls= ob.type();				
	set<Uml::Attribute> attrs=cls.attributes();		
	
	// Adding parent attributes
	set<Uml::Attribute> aattrs=Uml::AncestorAttributes(cls);
	attrs.insert(aattrs.begin(),aattrs.end());

	for(set<Uml::Attribute>::iterator ai = attrs.begin();ai != attrs.end(); ai++) 
	{
		if(string(ai->type())=="String")
		{
			string str=ai->name();
			if(str=="name")
			{
				string value=ob.getStringAttr(*ai);
				if(value.empty())value="<empty string>";
				return value;
			}			
		}				
	}	
	return string("<no name specified>");
}
#endif
