/*
 *	Copyright (c) Vanderbilt University, 2001-2002
 *	ALL RIGHTS RESERVED
 *
 *	Vanderbilt University disclaims all warranties with regard to this
 *	software, including all implied warranties of merchantability
 *	and fitness.  In no event shall Vanderbilt University be liable for
 *	any special, indirect or consequential damages or any damages
 *	whatsoever resulting from loss of use, data or profits, whether
 *	in an action of contract, negligence or other tortious action,
 *	arising out of or in connection with the use or performance of
 *	this software.
 *
 *  FILE NAME:	Library Update 
 *	SYNOPSIS:	Class that updates References from one library to another 
 *  AUTHOR:		Aditya Agrawal - aditya,agrawal@vanderbilt.edu
 *  CREATED:	08/24/03
 *  LAST UPDATE: 
 */

#ifndef LIBRARY_UPDATE_H
#define LIBRARY_UPDATE_H


#ifdef GME_INTERPRETER_USED
#define MESSAGE_OUT(STR) AfxMessageBox(STR)
#else
#define MESSAGE_OUT(STR) cout << STR << endl
#endif


#include "iostream"
#include "Uml.h"
#include "UdmBase.h"
#include "UMLModelTransformer.h"
#include "UmlExt.h"
#include <string>

using namespace std;

#include <map>
//#include <hash_map>

/*
struct udmhash
{
	size_t operator()(const Udm::Object __s) const { return __s.uniqueId(); }
	//bool operator()(const Udm::Object& first) const {return first.uniqueId();}
};*/

/*
template<class Key, class Traits = less<Key> >
   class udmhash
   {
   Traits comp;
public:
   const static size_t bucket_size = 4;
   const static size_t min_buckets = 8;
   udmhash( );
   udmhash( Traits pred ) {}
   size_t operator( )( const Key& Key ) const {return Key.uniqueId()}
   bool operator( )( 
      const Key& _Key1,
      const Key& _Key2
	  ) const {return (_Key1.uniqueId() <= _Key2.uniqueId());}
   };*/

//typedef stdext::hash_map<string, UMLModelTransformer::Class> String2ClassMap;
//typedef stdext::hash_map<UMLModelTransformer::Class, UMLModelTransformer::Class, equal_to<UMLModelTransformer::Class>, udmhash> Class2ClassMap;
//typedef stdext::hash_map<UMLModelTransformer::Class, UMLModelTransformer::Class, udmhash, equal_to<UMLModelTransformer::Class> > Class2ClassMap;

typedef std::map<string, UMLModelTransformer::Class> String2ClassMap;
typedef std::map<UMLModelTransformer::Class, UMLModelTransformer::Class > Class2ClassMap;
//typedef map<UMLModelTransformer::Class, UMLModelTransformer::Class, udmhash, equal_to<UMLModelTransformer::Class> > Class2ClassMap;

class UMTContainer
{
public:
	UMTContainer(Udm::Object Obj) : obj(Obj) {meta = Obj.type();}
	Udm::ChildrenAttr<UMLModelTransformer::ClassDiagram> const ClassDiagram_kind_children()
	{	if((string)meta.name() == "RootFolder")
			return Udm::ChildrenAttr<UMLModelTransformer::ClassDiagram>(UMLModelTransformer::RootFolder::Cast(obj).ClassDiagram_kind_children());
		else
			return UMLModelTransformer::Package::Cast(obj).ClassDiagram_kind_children();
	}

	Udm::StringAttr name() const 
	{	if((string)meta.name() == "RootFolder")
			return UMLModelTransformer::RootFolder::Cast(obj).name();
		else
			return UMLModelTransformer::Package::Cast(obj).name();
	}
	
	Udm::Object obj;
protected:
	
	Uml::Class meta;

};


class LibraryUpdate
{
public:
	LibraryUpdate(Udm::DataNetwork &Umt, Udm::Object OldLib, Udm::Object NewLib) : umt(Umt), oldLib(OldLib), newLib(NewLib){}
	void update();

protected:
	Udm::DataNetwork &umt;
	UMTContainer oldLib, newLib;

	//UMLModelTransformer::RootFolder grf;
	
	String2ClassMap oldClassMap;
	Class2ClassMap newClassMap;
	
	void Phase1_BuildMaps();
		void BuildClassMap(UMTContainer library, String2ClassMap &map);
		void BuildOldNewMap(UMTContainer library, String2ClassMap &oldM, Class2ClassMap &newM);
		
	void Phase2_TransferReferences();
		void TransferReferences(UMLModelTransformer::Transformation trf);
		void TransferReferences(UMLModelTransformer::Expression rule);
		void TransferReferences(UMLModelTransformer::CompoundRule rule);
		void TransferReferences(UMLModelTransformer::Test test);
		void TransferReferences(UMLModelTransformer::PrimitiveRule rule);
		
		void TransferReferences(UMLModelTransformer::Package pack);
		void TransferReferences(UMLModelTransformer::ClassDiagram cd);

		vector<UMLModelTransformer::ClassDiagram> GetClassDiagrams( vector<UMLModelTransformer::Namespace> &nss );
		string GetQualifiedName(Udm::Object &obj); //Added for handling namespaces

};

#endif